/****************************************/
/*  CompareFunc.c body          	*/
/*					*/
/*  Description : comp functions for    */
/*                gtk_clist using       */
/*		  attached  FileData    */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/*****************************************************/
/* if string1 come first in the dictonary order then */
/* it return true else false ( or contrary ).        */
/* the maximum string length is 2048                 */
static gboolean cmp_string(gchar *string1, gchar *string2)
{ guint16 compt;
  for(compt=0;compt<2048 && string1[compt] && string2[compt];compt++)
  {
    if(string1[compt]>string2[compt]) return(TRUE);
    else if(string2[compt]>string1[compt]) return(FALSE);
  }
  if(string1[compt]) return(TRUE); else return(FALSE);
}
/*****************************************************/

/***************************************/
/* function that made all sort by name */
static gint all_sort_by_name(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{ FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)   
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)cmp_string(vFileData1->name,vFileData2->name));
      else
        return((gint)cmp_string(vFileData1->name,vFileData2->name));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)cmp_string(vFileData1->name,vFileData2->name));
  else
    return((gint)cmp_string(vFileData1->name,vFileData2->name));
}			
/***************************************/

/*****************************************/
/* function that made all sort by rights */
static gint all_sort_by_rights(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)((vFileData1->stat.st_mode & 0x1FF) < (vFileData2->stat.st_mode & 0x1FF)));
      else
        return((gint)((vFileData1->stat.st_mode & 0x1FF) < (vFileData2->stat.st_mode & 0x1FF)));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)((vFileData1->stat.st_mode & 0x1FF) < (vFileData2->stat.st_mode & 0x1FF)));
  else
    return((gint)((vFileData1->stat.st_mode & 0x1FF) < (vFileData2->stat.st_mode & 0x1FF)));
}
/*****************************************/

/***************************************/
/* function that made all sort by size */
static gint all_sort_by_size(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->stat.st_size > vFileData2->stat.st_size));
      else
        return((gint)(vFileData1->stat.st_size > vFileData2->stat.st_size));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->stat.st_size > vFileData2->stat.st_size));
  else
    return((gint)(vFileData1->stat.st_size > vFileData2->stat.st_size));
}
/***************************************/

/***************************************/
/* function that made all sort nothing */
static gint all_sort_none(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(0);
      else
        return(1);
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(0);
  else
    return(1);
}
/***************************************/

/**********************************************/
/* function that made all sort by access date */
static gint all_sort_by_access_date(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->stat.st_atime > vFileData2->stat.st_atime));
      else
        return((gint)(vFileData1->stat.st_atime > vFileData2->stat.st_atime));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->stat.st_atime > vFileData2->stat.st_atime));
  else
    return((gint)(vFileData1->stat.st_atime > vFileData2->stat.st_atime));
}
/**********************************************/

/*********************************************/
/* function that made all sort by modif date */
static gint all_sort_by_modif_date(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->stat.st_mtime > vFileData2->stat.st_mtime));
      else
        return((gint)(vFileData1->stat.st_mtime > vFileData2->stat.st_mtime));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->stat.st_mtime > vFileData2->stat.st_mtime));
  else
    return((gint)(vFileData1->stat.st_mtime > vFileData2->stat.st_mtime));
}
/*********************************************/

/**********************************************/
/* function that made all sort by change date */
static gint all_sort_by_change_date(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->stat.st_ctime > vFileData2->stat.st_ctime));
      else
        return((gint)(vFileData1->stat.st_ctime > vFileData2->stat.st_ctime));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->stat.st_ctime > vFileData2->stat.st_ctime));
  else
    return((gint)(vFileData1->stat.st_ctime > vFileData2->stat.st_ctime));
}
/**********************************************/

/****************************************/
/* function that made all sort by owner */
static gint all_sort_by_owner(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->stat.st_uid > vFileData2->stat.st_uid));
      else
        return((gint)(vFileData1->stat.st_uid > vFileData2->stat.st_uid));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->stat.st_uid > vFileData2->stat.st_uid));
  else
    return((gint)(vFileData1->stat.st_uid > vFileData2->stat.st_uid));
}
/****************************************/

/****************************************/
/* function that made all sort by group */
static gint all_sort_by_group(GtkCList *clist,gconstpointer ptr1,
  gconstpointer ptr2,gboolean dir_first,gboolean invert)
{
 FileData *vFileData1, *vFileData2;

  vFileData1 = ((GtkCListRow *)ptr1)->data;
  vFileData2 = ((GtkCListRow *)ptr2)->data;

  /* "." and ".." MUST ALWAYS be on the top */    
  if(!strcmp(vFileData1->name,"."))  return(0);
  if(!strcmp(vFileData2->name,"."))  return(1);
  if(!strcmp(vFileData1->name,"..")) return(0);
  if(!strcmp(vFileData2->name,"..")) return(1);

  if(dir_first == TRUE)
  {
    if((S_ISDIR(vFileData1->stat.st_mode) ||  
        (S_ISLNK(vFileData1->stat.st_mode) && 
	 !(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData1->link_stat->st_mode)) ) &&
       (S_ISDIR(vFileData2->stat.st_mode) ||  
        (S_ISLNK(vFileData2->stat.st_mode) && 
	 !(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData2->link_stat->st_mode)) ))	 
    {
      if(invert == TRUE)
        return(!(gint)(vFileData1->stat.st_gid > vFileData2->stat.st_gid));
      else
        return((gint)(vFileData1->stat.st_gid > vFileData2->stat.st_gid));
    }  
    else
    if(S_ISDIR(vFileData1->stat.st_mode) ||  
       (S_ISLNK(vFileData1->stat.st_mode) && 
	!(vFileData1->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData1->link_stat->st_mode)) )
      return(0);
    else
    if(S_ISDIR(vFileData2->stat.st_mode) ||  
       (S_ISLNK(vFileData2->stat.st_mode) && 
	!(vFileData2->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	S_ISDIR(vFileData2->link_stat->st_mode)) )
      return(1);
  }
  if(invert == TRUE)
    return(!(gint)(vFileData1->stat.st_gid > vFileData2->stat.st_gid));
  else
    return((gint)(vFileData1->stat.st_gid > vFileData2->stat.st_gid));
}
/****************************************/


gint sort_none_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_name_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,FALSE,FALSE));}
gint sort_by_owner_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_group_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_rights_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_size_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_access_date_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_modif_date_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,FALSE,FALSE)); }
gint sort_by_change_date_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,FALSE,FALSE)); }


gint sort_none_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_name_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,FALSE,TRUE));}
gint sort_by_owner_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_group_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_rights_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_size_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_access_date_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_modif_date_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,FALSE,TRUE)); }
gint sort_by_change_date_inv_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,FALSE,TRUE)); }


gint sort_none_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_name_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,TRUE,FALSE));}
gint sort_by_owner_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_group_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_rights_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_size_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_access_date_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_modif_date_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,TRUE,FALSE)); }
gint sort_by_change_date_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,TRUE,FALSE)); }


gint sort_none_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_none(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_name_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_name(clist,ptr1,ptr2,TRUE,TRUE));}
gint sort_by_owner_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_owner(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_group_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_group(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_rights_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_rights(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_size_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_size(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_access_date_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_access_date(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_modif_date_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_modif_date(clist,ptr1,ptr2,TRUE,TRUE)); }
gint sort_by_change_date_inv_dir_first_func(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{ return(all_sort_by_change_date(clist,ptr1,ptr2,TRUE,TRUE)); }
