/****************************************/
/*  FileData.c headers         	        */
/*					*/
/*  Description : Datas attached t each */
/*		  file. 		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#ifndef __FILE_DATA_H__
#define __FILE_DATA_H__

typedef struct _FileData FileData;


/* theses define are for the extra_info  */
#define FILE_DATA_IS_EXECUTABLE    1<<0 /* is this file executable for somebody */
#define FILE_DATA_IS_ALLOW_TO_EXEC 1<<1 /* is this file executable for the current user */
#define FILE_DATA_IS_BROKEN_SYMB   1<<2 /* can't have info on what the link go */
                                        /* (what ever is the reason)           */
/* and specialy for Trees. If set, this mean that the content of
   this directory have been listed */
#define FILE_DATA_IS_DIR_SCAN      1<<3

/* theses datas are attached to each row */
/* THIS MUST BE USED FOR DOING STUFF     */
struct _FileData
{
  gchar *path;            /* directory in which the file is */
  gchar *name;            /* the file name                  */
  struct stat stat;       /* create by using lstat          */
  struct stat *link_stat; /* create by using stat if the current file */
                          /* is a link (to have destination rights)   */
  FileType *type;         /* the FileType corresponding to the file   */
                          /* or NULL if not set                       */
  gchar *link_to_file;    /* name of the file where the link go. NULL */
                          /* if not a link                            */
  gint  extra_info;
  gint  ref_count;        /* number of pointer on it. When null, this */
  			  /* structure must be delete.                */
};

/* free a FileData          */
/* the file type is not free*/
void file_data_free(FileData *pFileData);

/* Return a copy of pFileData  */
FileData *file_data_copy(FileData *pFileData);

/* Return TRUE if p1FileData and p2FileData   */
/* indicate the same file. Else FALSE.        */
gboolean file_data_is_equal(FileData *p1FileData,FileData *p2FileData);

#define FILE_DATA_LSTAT_ERROR 1 /* error while using lstat on file */

/* create a new file data filled with the info of file "file_name"  */
/* in directory "file_path" (file_path is ALWAYS ended by a /).     */
/* The corresponding file_type is attached to the file if pFileType */
/* is not NULL (pFileType is a list of all the define file_type)    */
/* pOwner is the list of all the known user on this computer, if    */
/* NULL then we don't check the FILE_DATA_IS_ALLOW_TO_EXEC          */
/* The FileData returned can be NULL and GFileError can be set to   */
/* FILE_DATA_LSTAT_ERROR.                                           */
FileData *file_data_new_from_file(gchar *file_path,gchar *file_name,
  FileTypeList *pFileTypeList,Owner *pOwner);

/* Find the filetype corresponding to pFileData */
/* and attach it to pFileData                   */
/* Return : The corresponding FileType.         */
FileType *file_data_find_type(FileTypeList *pFileTypeList, FileData *pFileData);

#define FILE_DATA_DIR_EACCES  1 /*Permission denied.*/
#define FILE_DATA_DIR_EMFILE  2 /*Too many file descriptors in use by process.*/
#define FILE_DATA_DIR_ENFILE  3 /*Too many files are currently open in the system.*/
#define FILE_DATA_DIR_ENOENT  4 /*Directory does not exist, or name is an empty string.*/
#define FILE_DATA_DIR_ENOMEM  5 /*Insufficient memory to complete the operation.*/
#define FILE_DATA_DIR_ENOTDIR 6 /*name is not a directory.*/
#define FILE_DATA_DIR_UNKNOWN 7 /* unknown error. */

/* Scan the content of a directory. Create a FileData for each      */
/* entries. For eached FileData, user_func is called with the       */
/* FileData and it own data. If user_func return FALSE the FileData */
/* is suppress.                                                     */
/* The corresponding file_type is attached to the file if pFileType */
/* is not NULL (pFileType is a list of all the define file_type)    */
/* pOwner is the list of all the known user on this computer, if    */
/* NULL then we don't check the FILE_DATA_IS_ALLOW_TO_EXEC.         */
/* If file_data_open_dir return FALSE, gfile_errno can be set to :  */
/*  FILE_DATA_DIR_EACCES  Permission denied.                                   */
/*  FILE_DATA_DIR_EMFILE  Too many file descriptors in use by process.         */
/*  FILE_DATA_DIR_ENFILE  Too many files are currently open in the system.     */
/*  FILE_DATA_DIR_ENOENT  Directory does not exist, or name is an empty string.*/
/*  FILE_DATA_DIR_ENOMEM  Insufficient memory to complete the operation.       */ 
/*  FILE_DATA_DIR_ENOTDIR name is not a directory.                             */
/*  FILE_DATA_DIR_UNKNOWN unknown error.                                       */
gboolean file_data_open_dir(gchar *dir_name,
  FileTypeList *pFileTypeList,Owner *pOwner,
  gboolean (* user_func)(FileData *pFileData,gpointer data),gpointer data);

#endif  /* __FILE_DATA_H__ */
