/****************************************/
/*  FileDataList.c body         	*/
/*					*/
/*  Description : List of FileData      */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/******************************/
/* Return a new FileDataList. */
FileDataList *file_data_list_new()
{ FileDataList *vFileDataList;

  vFileDataList = g_malloc(sizeof(FileDataList));
  vFileDataList->list =  g_list_alloc();
  vFileDataList->last = vFileDataList->list;
  return(vFileDataList); 
}
/******************************/

/***************************************************************/
/* Return a new FileDataList which is a copy of pFileDataList. */
FileDataList *file_data_list_copy(FileDataList *pFileDataList)
{ FileDataList *vFileDataList;

  vFileDataList = file_data_list_new();
  file_data_list_init(pFileDataList);
  while(file_data_list_is_end(pFileDataList) == FALSE)
  {
    file_data_list_append(vFileDataList, file_data_list_current(pFileDataList));
    file_data_list_next(pFileDataList);
  }  
  return(vFileDataList);
}
/***************************************************************/

/*****************************************/
/* Append a FileData to the FileDataList */
/* We keep A COPY of pFileData           */
void file_data_list_append(FileDataList *pFileDataList, FileData *pFileData)
{ GList    *new;

  new = g_malloc(sizeof(GList));
  new->next = NULL; new->prev = pFileDataList->last;
  new->data = file_data_copy(pFileData);
  pFileDataList->last->next = new;
  pFileDataList->last = new;
}
/*****************************************/

/********************************************/
/* Free the FileDataList. 		    */
void file_data_list_free(FileDataList *pFileDataList)
{
  file_data_list_init(pFileDataList);
  while(file_data_list_is_end(pFileDataList) == FALSE)
  {
    file_data_free(file_data_list_current(pFileDataList));
    file_data_list_next(pFileDataList);
  }
  g_list_free(pFileDataList->list);
  g_free(pFileDataList);
}
/********************************************/

/*********************************************************/
/************ FOR GOING OVER ALL THE FILE DATAS **********/

/**********************************************************/
/* After a call to this procedure, file_data_list_current */
/* return the first FileData                              */
void file_data_list_init(FileDataList *pFileDataList)
{
  pFileDataList->current = g_list_next(pFileDataList->list);
}
/**********************************************************/

/*********************************/
/* Return the FileData we are on */
FileData *file_data_list_current(FileDataList *pFileDataList)
{
  return((FileData *)pFileDataList->current->data);
}
/*********************************/

/***************************/
/* Go to the next FileData */
void file_data_list_next(FileDataList *pFileDataList)
{
  pFileDataList->current = g_list_next(pFileDataList->current);
}
/***************************/

/************************************/
/* say if there is no more FileData */
gboolean file_data_list_is_end(FileDataList *pFileDataList)
{
  return(pFileDataList->current == NULL);
}
/************************************/

/*********************************************************/
