/****************************************/
/*  FileTypeAutomate.c body             */
/*					*/
/*  Description : FileTypeAutomate take */
/*   care of the user defined file type */
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

/*#include "GFile.h"*/
#include <gnome.h>

/*********************************************************/
/* A type can be define using all ASCII. '*' and '?' are */
/* replaced by : '*' everything or nothing               */
/*               '?' one character                       */
/* example : is_of_type("*.gif","file.gif") return TRUE  */
/*           is_of_type("?.gif","file.gif") return FALSE */
/*            because "file" as 4 charcaters not only one*/
/* Return TRUE if file_name is of type type_str.         */
gboolean is_of_type(gchar *type_str, gchar *file_name)
{ guint pos = 0;
  guint last_pos = -1;
  guint last_type_pos = -1;  
  guint type_pos = 0;

  /* security check */  
  if((type_str == NULL) || (file_name == NULL)) return(FALSE);
  
  while(file_name[pos])
  {
    switch(type_str[type_pos])
    {
      case  0  : 
        if(last_type_pos != -1)
	{
	  pos = ++last_pos; type_pos = last_type_pos;
        } else return(FALSE);
	break;
      case '*' :
        last_type_pos = ++type_pos;
	last_pos = pos;
        break;
      case '?' :
        pos++; type_pos++;
        break;
      default  :
        if(type_str[type_pos] == file_name[pos])
	{
	  pos++; type_pos++;
	} else if(last_type_pos != -1)
	{ 
	  pos = ++last_pos; type_pos = last_type_pos; 
	} else return(FALSE);
        break;
    }
  }
  while((type_str[type_pos]) && (type_str[type_pos] == '*')){type_pos++;}
  /* if we are at the end of type_str then we have reconize the file_name */
  return(!type_str[type_pos]);
}
/*********************************************************/

/*********************************************************/
/* Return TRUE if file_name is of type type_str. type_str*/
/* is a string of several type (like for is_of_type)     */
/* separated by ';'                                      */
gboolean is_of_multi_type(gchar *type_str, gchar *file_name)
{ guint pos = 0;
  guint last_pos = 0;
  gboolean return_value = FALSE;
  gchar *tmp_string;

  /* security check */  
  if((type_str == NULL) || (file_name == NULL)) return(FALSE);

  while(return_value == FALSE)
  {
    for(pos = last_pos; (type_str[pos]) && (type_str[pos] != ';'); pos++);
    if(type_str[pos]) {
      tmp_string = g_strndup(&type_str[last_pos],pos-last_pos);
      return_value = is_of_type(tmp_string,file_name);
      g_free(tmp_string);
      last_pos = pos+1;
    } else {
      return(is_of_type(&type_str[last_pos],file_name));
    }
  }
}
/*********************************************************/
