/****************************************/
/*  GFileList.c body         		*/
/*					*/
/*  Description : Manage the file list  */
/*		  widget		*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

/**********************************************/
/* A FAIRE : 			              */
/* - probleme de la taille minimum et de la   */
/*   taille de preference au demarrage.       */
/* - pouvoir changer a chaux les preferences  */
/* - notation americaine de la date           */
/* - la touche enter doit faire comme un clic */
/* - verifier tout les codes d'erreurs des    */
/*   fonctions C.                             */
/* - separer le cb event du menu popup        */
/* - faire que la GFileList marche meme si    */
/*   owner_list, group_list et file_type_list */
/*   sont NULL comme dans DefaultGFileListPref*/
/* - appel aux programmes externes plus propre*/
/* - drag&drop de fichier.                    */
/* - Insertion, suppression de colonne        */
/*   en cliquant button 3 sur les titres.     */
/* - gerer la localisation.                   */
/* - verifier qu'un fichier n'est pas deja    */
/*   present avant l'insertion => modifier    */
/* - Optimiser un peu le code (pour beaucoup  */
/*   plus tard)                               */
/**********************************************/

#include "GFile.h"

/* can be used for external use */
guint gfilelist_get_type();
void gfilelist_set_sort(GFileList *pGFileList, guint8 sort_mode);
GtkWidget* gfilelist_new (GFileListPref pGFileListPref);
void gfilelist_free(GFileList *pGFileList);
gboolean gfilelist_add_file(GFileList *pGFileList,FileData *pFileData);
void gfilelist_add_file_by_name(GFileList *pGFileList,gchar *name);
void gfilelist_add_file_by_full_name(GFileList *pGFileList,gchar *full_name);
void gfilelist_remove_file_with_check(GFileList *pGFileList,FileData *pFileData);
void gfilelist_remove_file(GFileList *pGFileList,FileData *pFileData);
void gfilelist_remove_all(GFileList *pGFileList);
void gfilelist_fill_status(GFileList *pGFileList);
gboolean gfilelist_open_dir(GFileList *pGFileList,gchar *name);
void gfilelist_set_filetype(GFileList *pGFileList,FileTypeList *pFileTypeList);
FileDataList *gfilelist_list_selected(GFileList *pGFileList);
void gfilelist_unselect_all(GFileList *pGFileList);
void gfilelistpref_copy(GFileListPref *pGFileListPref1,
  GFileListPref *pGFileListPref2);

/* internal use only */
static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args);
static void gfilelist_class_init(GFileListClass *class);
static void gfilelist_init (GFileList *pGFileList);
static gchar *gfilelist_date_2_string(GFileList *pGFileList,time_t date);  
static void gfile_list_rights_2_string(mode_t mode,gchar string[]);
static void down_dir(gchar *path,gchar *pathdest);
static void gfilelist_row_select_cb(GtkCList *clist,gint row,gint column, GdkEvent
  *event,GFileList *pGFileList);
static void gfilelist_row_unselect_cb(GtkCList *clist,gint row,gint column, GdkEvent
  *event,GFileList *pGFileList);
static gint gfilelist_event_cb(GtkCList *clist, GdkEvent *event,GFileList
  *pGFileList);

static void prop_menu_properties_cb(GtkWidget *Widget,GFileList *pGFileList);
		
/* The default Preference */
GFileListPref DefaultGFileListPref =
{
  FALSE,       /* don't hide the clist title */
  "Name",0,60,
  "Owner",1,40,
  "Group",2,40,
  "Rights",3,60,
  "Size",4,40,
  "Access date",5,70,
  "Modif date",6,70,
  "Change date",7,70,  
  18,
  100,100,
  TRUE,		/* allow the status bar */
  129,		/* sort mode */
  TRUE,         /* allow the right click menu */
  FALSE,        /* show hidden files */
  TRUE,         /* lynx like motion  */
  NULL,         /* owner_list can't have a default value */
  NULL,         /* group_list can't have a default value */
  NULL          /* file_type_list can't have a default value */
}; 

/*************************************************************/
guint gfilelist_get_type ()
{
  static guint gfilelist_type = 0;

  if (!gfilelist_type)
  {
    GtkTypeInfo gfilelist_info =
    {
       "GFileList",
       sizeof (GFileList),
       sizeof (GFileListClass),
       (GtkClassInitFunc)  gfilelist_class_init,
       (GtkObjectInitFunc) gfilelist_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     gfilelist_type = gtk_type_unique(gtk_vbox_get_type(), &gfilelist_info);
  }
  return gfilelist_type;
}
/*************************************************************/

/*************************************************************/
enum { UP_DIR_SIGNAL, DOWN_DIR_SIGNAL, SCAN_DIR_SIGNAL, 
       FILE_MODE_CHANGED_SIGNAL, LAST_SIGNAL};
static gint gfilelist_signals[LAST_SIGNAL] = { 0 };

static void string_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*string_marshal_func)(GFileList *, gchar *,gpointer);
    string_marshal_func rfunc = (string_marshal_func) func;
    (*rfunc) (GFILELIST(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void two_file_data_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_file_data_marshal_func)(GFileList *, FileData *, FileData *,gpointer);
    two_file_data_marshal_func rfunc = (two_file_data_marshal_func) func;
    (*rfunc) (GFILELIST(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]),func_data );
}

static void file_data_list_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*file_data_list_marshal_func)(GFileList *, FileDataList *, gpointer);
    file_data_list_marshal_func rfunc = (file_data_list_marshal_func) func;
    (*rfunc) (GFILELIST(object),GTK_VALUE_POINTER(args[0]),func_data);
}

static void gfilelist_class_init(GFileListClass *class)
{
/* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;
       
  gfilelist_signals[UP_DIR_SIGNAL] = gtk_signal_new ("up_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,up_dir),
            string_marshal, GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);

  gfilelist_signals[DOWN_DIR_SIGNAL] = gtk_signal_new ("down_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,down_dir),
            gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

  gfilelist_signals[SCAN_DIR_SIGNAL] = gtk_signal_new ("scan_dir",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,scan_dir),
            file_data_list_marshal, GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);

  gfilelist_signals[FILE_MODE_CHANGED_SIGNAL] = gtk_signal_new
            ("file_mode_changed",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (GFileListClass,file_mode_changed),
            two_file_data_marshal, GTK_TYPE_NONE, 2, GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);

   gtk_object_class_add_signals (object_class, gfilelist_signals, LAST_SIGNAL);
   class->up_dir = NULL; class->down_dir = NULL; class->scan_dir = NULL;
   class->file_mode_changed = NULL;
}
/*************************************************************/

/* copy the content of  pGFileListPref2 in pGFileListPref1 */
void gfilelistpref_copy(GFileListPref *pGFileListPref1,
  GFileListPref *pGFileListPref2)
{
  memcpy(pGFileListPref1,pGFileListPref2,sizeof(GFileListPref));
  pGFileListPref1->name_title = g_strdup(pGFileListPref2->name_title);
  pGFileListPref1->owner_title = g_strdup(pGFileListPref2->owner_title);
  pGFileListPref1->group_title = g_strdup(pGFileListPref2->group_title);
  pGFileListPref1->rights_title = g_strdup(pGFileListPref2->rights_title);
  pGFileListPref1->size_title = g_strdup(pGFileListPref2->size_title);
  pGFileListPref1->last_access_date_title = g_strdup(pGFileListPref2->last_access_date_title);
  pGFileListPref1->last_modif_date_title = g_strdup(pGFileListPref2->last_modif_date_title);
  pGFileListPref1->last_status_change_date_title = g_strdup(pGFileListPref2->last_status_change_date_title);
}  

/*********************/
/* set the sort mode */
void gfilelist_set_sort(GFileList *pGFileList, guint8 sort_mode)
{
  switch(sort_mode)
  {
    case SORT_NONE           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_func); break;    
    case SORT_BY_NAME        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_func); break;    
    case SORT_BY_OWNER       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_func); break;    
    case SORT_BY_GROUP       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_func); break;    
    case SORT_BY_RIGHTS      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_func); break;
    case SORT_BY_SIZE        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_func); break;    
    case SORT_BY_ACCESS_DATE :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_func); break;        
    case SORT_BY_MODIF_DATE  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_func); break;    
    case SORT_BY_CHANGE_DATE   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_func); break;    

    case SORT_NONE | SORT_INVERT_ORDER           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_inv_func); break;    
    case SORT_BY_NAME | SORT_INVERT_ORDER        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_inv_func); break;    
    case SORT_BY_OWNER | SORT_INVERT_ORDER       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_inv_func); break;    
    case SORT_BY_GROUP | SORT_INVERT_ORDER       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_inv_func); break;    
    case SORT_BY_RIGHTS | SORT_INVERT_ORDER      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_inv_func); break;
    case SORT_BY_SIZE | SORT_INVERT_ORDER        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_inv_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_INVERT_ORDER :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_inv_func); break;        
    case SORT_BY_MODIF_DATE | SORT_INVERT_ORDER  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_inv_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_INVERT_ORDER   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_inv_func); break;

    case SORT_NONE | SORT_DIR_FIRST          :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_dir_first_func); break;    
    case SORT_BY_NAME | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_dir_first_func); break;    
    case SORT_BY_OWNER | SORT_DIR_FIRST       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_dir_first_func); break;    
    case SORT_BY_GROUP | SORT_DIR_FIRST       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_dir_first_func); break;    
    case SORT_BY_RIGHTS | SORT_DIR_FIRST      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_dir_first_func); break;
    case SORT_BY_SIZE | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_dir_first_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_DIR_FIRST :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_dir_first_func); break;        
    case SORT_BY_MODIF_DATE | SORT_DIR_FIRST  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_dir_first_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_DIR_FIRST   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_dir_first_func); break;    

    case SORT_NONE | SORT_INVERT_ORDER | SORT_DIR_FIRST           :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_none_inv_dir_first_func); break;    
    case SORT_BY_NAME | SORT_INVERT_ORDER | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_name_inv_dir_first_func); break;    
    case SORT_BY_OWNER | SORT_INVERT_ORDER | SORT_DIR_FIRST       :  
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_owner_inv_dir_first_func); break;    
    case SORT_BY_GROUP | SORT_INVERT_ORDER | SORT_DIR_FIRST       :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_group_inv_dir_first_func); break;    
    case SORT_BY_RIGHTS | SORT_INVERT_ORDER | SORT_DIR_FIRST      :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_rights_inv_dir_first_func); break;
    case SORT_BY_SIZE | SORT_INVERT_ORDER | SORT_DIR_FIRST        :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_size_inv_dir_first_func); break;    
    case SORT_BY_ACCESS_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_access_date_inv_dir_first_func); break;        
    case SORT_BY_MODIF_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST  :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_modif_date_inv_dir_first_func); break;    
    case SORT_BY_CHANGE_DATE | SORT_INVERT_ORDER | SORT_DIR_FIRST   :
      gtk_clist_set_compare_func(GTK_CLIST(pGFileList->Clist),
  	sort_by_change_date_inv_dir_first_func); break;	    
  }
  pGFileList->Preference.sort_mode = sort_mode;
  gtk_clist_sort(GTK_CLIST(pGFileList->Clist));
}
/*********************/

/****************************************************************/
/* get the sort mode used (usefull because users can changed is */
/* by clicking on the titles)                                   */
guint8 gfilelist_get_sort(GFileList *pGFileList)
{
  return(pGFileList->Preference.sort_mode);
}
/****************************************************************/

/**************************************************************/
/* Function called eached time a titles of the list is cliked */
static void title_cb(GtkCList *clist,gint column,GFileList *pGFileList)
{ guint8 sort_mode, short_last_mode;

  short_last_mode = pGFileList->Preference.sort_mode;

  /* what ever happend, keep the dir first choice */
  if(pGFileList->Preference.sort_mode & SORT_DIR_FIRST){
    sort_mode = SORT_DIR_FIRST;
    short_last_mode -= SORT_DIR_FIRST;
  } else
    sort_mode = 0;
  
  if(short_last_mode & SORT_INVERT_ORDER)
    short_last_mode -= SORT_INVERT_ORDER;

  if((guint8)column == pGFileList->Preference.name_pos)
  { /*g_print("name sort\n");*/
    if((short_last_mode == SORT_BY_NAME) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_NAME);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_NAME);
  } else
  if((guint8)column == pGFileList->Preference.owner_pos)
  { /*g_print("owner sort\n");*/
    if((short_last_mode == SORT_BY_OWNER) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_OWNER);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_OWNER);
  } else
  if((guint8)column == pGFileList->Preference.group_pos)
  { /*g_print("group sort\n");*/
    if((short_last_mode == SORT_BY_GROUP) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_GROUP);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_GROUP);
  } else
  if((guint8)column == pGFileList->Preference.rights_pos)
  { /*g_print("rights sort\n");*/
    if((short_last_mode == SORT_BY_RIGHTS) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_RIGHTS);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_RIGHTS);
  } else
  if((guint8)column == pGFileList->Preference.size_pos)
  { /*g_print("size sort\n");*/
    if((short_last_mode == SORT_BY_SIZE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_SIZE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_SIZE);
  } else
  if((guint8)column == pGFileList->Preference.last_access_date_pos)
  { /*g_print("last access date sort\n");*/
    if((short_last_mode == SORT_BY_ACCESS_DATE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_ACCESS_DATE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_ACCESS_DATE);
  } else
  if((guint8)column == pGFileList->Preference.last_modif_date_pos)
  { /*g_print("last modif date sort\n");*/
    if((short_last_mode == SORT_BY_MODIF_DATE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_MODIF_DATE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_MODIF_DATE);
  } else
  if((guint8)column == pGFileList->Preference.last_status_change_date_pos)
  { /*g_print("last status change date sort\n");*/
    if((short_last_mode == SORT_BY_CHANGE_DATE) &&
       !(pGFileList->Preference.sort_mode &  SORT_INVERT_ORDER))
      gfilelist_set_sort(pGFileList, sort_mode | SORT_INVERT_ORDER | SORT_BY_CHANGE_DATE);
    else      
      gfilelist_set_sort(pGFileList, sort_mode | SORT_BY_CHANGE_DATE);
  }

}
/**************************************************************/

/********************************/
/* called when a key is pressed */
#define ENTER              65293
#define LEFT_ARROW         65361
#define RIGHT_ARROW        65363
#define BACKSPACE          65288
#define KEYPAD_LEFT_ARROW  65430
#define KEYPAD_RIGHT_ARROW 65432
gint key_press_event_cb(GtkWidget *widget,GdkEventKey *event,GFileList *pGFileList)
{ gchar *tempbuf;
  gchar *dir_name = NULL;
  GList *select_row;
  FileData *vFileData;

/*  g_print("keypressed : %d = %c\n",event->keyval);*/
  /* Lynx motion not activated, exit */
  if((pGFileList->Preference.lynx_motion == FALSE) &&
     ((event->keyval == LEFT_ARROW) || (event->keyval == RIGHT_ARROW) ||
      (event->keyval == KEYPAD_LEFT_ARROW) || (event->keyval == KEYPAD_RIGHT_ARROW)))
    return(FALSE);

  /* if left arrow or backspace */
  if((event->keyval == LEFT_ARROW) || (event->keyval == KEYPAD_LEFT_ARROW) ||
     (event->keyval == BACKSPACE))
  {
down_etiq:
    tempbuf = g_malloc(strlen(pGFileList->current_path)+1);
    /* find the parent directory name */
    down_dir(pGFileList->current_path,tempbuf);
    /* if we are not at the root, suppress the last '/' */
    if(tempbuf[1] != 0)
      tempbuf[strlen(tempbuf)-1] = 0;	 
    gfilelist_open_dir(pGFileList,tempbuf);
    g_free(tempbuf);

    /* emit the corresponding signal */
    gtk_signal_emit(GTK_OBJECT(pGFileList),gfilelist_signals[DOWN_DIR_SIGNAL]);

    return(TRUE);
  }
  /* if right arrow */
  if((event->keyval == RIGHT_ARROW) || (event->keyval == KEYPAD_RIGHT_ARROW) ||
     (event->keyval == ENTER))
  {
    /* place select_row on the selected line list */
    select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
    /* if none selected or more than one selected, do nothing */
    if(!select_row || g_list_next(select_row))
      return(TRUE);
  
    /* get the data attach to the selected line */
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
    /* if is it not a directory or link on a directory, do nothing */
    if(!(S_ISDIR(vFileData->stat.st_mode) || 
         (S_ISLNK(vFileData->stat.st_mode) &&
          !(vFileData->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
          S_ISDIR(vFileData->link_stat->st_mode))))
      return(TRUE);

    /* if we are one .., goto parent directory */
    if(!strcmp(vFileData->name,".."))
      goto down_etiq;
    
    /* if we are on ., rescan the current directory */
    if(!strcmp(vFileData->name,".")) {
      tempbuf = g_strdup(pGFileList->current_path);
      /* if we are not at "/" suppress the last '/' */
      if(tempbuf[1] != 0)
        tempbuf[strlen(tempbuf)-1] = 0;
    } else {
      tempbuf = g_strconcat(vFileData->path,vFileData->name,NULL);
      dir_name = g_strdup(vFileData->name);
    }

    gfilelist_open_dir(pGFileList,tempbuf);
    g_free(tempbuf);
    /* let the others known that we go to the upper directory
       temp_dir_name */
    if(dir_name) /* only if it is not a reload */
    {
      gtk_signal_emit(GTK_OBJECT(pGFileList),gfilelist_signals[UP_DIR_SIGNAL],
        dir_name);
      g_free(dir_name);
    }
    return(TRUE);
  }
  /* event not taken in account by this function */
  return(FALSE);
}
/********************************/

/* use as a parameter given by gfilelist_new to gfilelist_init */
GFileListPref current_GFileListPref;

/*************************************************************/
static void gfilelist_init (GFileList *pGFileList)
{
  gint  nb_col = 0;
  gchar	*titles[9];

  GtkWidget *menu_items; /* for popup menu items */
  GtkWidget *scrolled_widget;
  GtkWidget *title_button; /* for replacing clist titles by buttons */
  GtkWidget *event_box;

  /* take parameters in account */
  gfilelistpref_copy(&(pGFileList->Preference),&current_GFileListPref);  
  /* free strings in the Preference */
  g_free(current_GFileListPref.name_title);
  g_free(current_GFileListPref.owner_title);
  g_free(current_GFileListPref.group_title); 
  g_free(current_GFileListPref.rights_title);  
  g_free(current_GFileListPref.size_title);
  g_free(current_GFileListPref.last_access_date_title); 
  g_free(current_GFileListPref.last_modif_date_title);
  g_free(current_GFileListPref.last_status_change_date_title);

  pGFileList->current_path = NULL;

  if(pGFileList->Preference.name_pos<255) 
  { titles[pGFileList->Preference.name_pos] = pGFileList->Preference.name_title; nb_col++; }
  if(pGFileList->Preference.owner_pos<255) 
  { titles[pGFileList->Preference.owner_pos] = pGFileList->Preference.owner_title; nb_col++; }
  if(pGFileList->Preference.group_pos<255) 
  { titles[pGFileList->Preference.group_pos] = pGFileList->Preference.group_title; nb_col++; }
  if(pGFileList->Preference.rights_pos<255) 
  { titles[pGFileList->Preference.rights_pos] = pGFileList->Preference.rights_title; nb_col++; }
  if(pGFileList->Preference.size_pos<255) 
  { titles[pGFileList->Preference.size_pos] = pGFileList->Preference.size_title; nb_col++; }
  if(pGFileList->Preference.last_access_date_pos<255) 
  { titles[pGFileList->Preference.last_access_date_pos] = pGFileList->Preference.last_access_date_title; nb_col++; }
  if(pGFileList->Preference.last_modif_date_pos<255) 
  { titles[pGFileList->Preference.last_modif_date_pos] = pGFileList->Preference.last_modif_date_title; nb_col++; }
  if(pGFileList->Preference.last_status_change_date_pos<255) 
  { titles[pGFileList->Preference.last_status_change_date_pos] = pGFileList->Preference.last_status_change_date_title; nb_col++; }

  /* The last one is empty */
  titles[nb_col++] = " ";

  /* Create the widget */ 
  pGFileList->Clist = gtk_clist_new_with_titles(nb_col ,titles);

  /* chose the selection mode (extended) */
  gtk_clist_set_selection_mode(GTK_CLIST(pGFileList->Clist), GTK_SELECTION_EXTENDED);
  /* ligne height */
  gtk_clist_set_row_height(GTK_CLIST(pGFileList->Clist), pGFileList->Preference.ligne_height);
  /* justify right the file size */
  if(pGFileList->Preference.size_pos<255)
    gtk_clist_set_column_justification(GTK_CLIST (pGFileList->Clist),
		 pGFileList->Preference.size_pos,GTK_JUSTIFY_RIGHT);

  gtk_signal_connect (GTK_OBJECT (pGFileList), 
		      "destroy",
		      (GtkSignalFunc) gfilelist_free,
		      pGFileList);

  gtk_signal_connect (GTK_OBJECT (pGFileList->Clist), 
		      "select_row",
		      (GtkSignalFunc) gfilelist_row_select_cb,
		      pGFileList);

  gtk_signal_connect (GTK_OBJECT (pGFileList->Clist), 
		      "unselect_row",
		      (GtkSignalFunc) gfilelist_row_unselect_cb,
		      pGFileList);

  /* Connect all titles */
  gtk_signal_connect(GTK_OBJECT(pGFileList->Clist), "click_column",
    GTK_SIGNAL_FUNC (title_cb), pGFileList);

  /* choose the order function */
  gfilelist_set_sort(pGFileList,pGFileList->Preference.sort_mode);

  /* hide the titles if we don't want to see them */
  if(pGFileList->Preference.hide_title)
    gtk_clist_column_titles_hide (GTK_CLIST(pGFileList->Clist));

  /* set the colonne size */
  if(pGFileList->Preference.name_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.name_pos,
                               pGFileList->Preference.name_width);
  if(pGFileList->Preference.owner_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.owner_pos,
                               pGFileList->Preference.owner_width);
  if(pGFileList->Preference.group_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.group_pos,
                               pGFileList->Preference.group_width);
  if(pGFileList->Preference.rights_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.rights_pos,
                               pGFileList->Preference.rights_width);
  if(pGFileList->Preference.size_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.size_pos,
                               pGFileList->Preference.size_width);
  if(pGFileList->Preference.last_access_date_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.last_access_date_pos,
                               pGFileList->Preference.last_access_date_width);
  if(pGFileList->Preference.last_modif_date_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.last_modif_date_pos,
                               pGFileList->Preference.last_modif_date_width);
  if(pGFileList->Preference.last_status_change_date_pos<255) 
    gtk_clist_set_column_width(GTK_CLIST(pGFileList->Clist),pGFileList->Preference.last_status_change_date_pos,
                               pGFileList->Preference.last_status_change_date_width);

  /* set the minimum size of the list ___________*/
  gtk_widget_set_usize(GTK_WIDGET(pGFileList->Clist),pGFileList->Preference.min_width,
                       pGFileList->Preference.min_height);

  /* define the scroll properties */
  scrolled_widget = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled_widget),
    GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_widget);
  gtk_container_add(GTK_CONTAINER(scrolled_widget), pGFileList->Clist);
  
  gtk_box_pack_start (GTK_BOX (pGFileList), scrolled_widget, TRUE, TRUE, 0);

  /* create a statusbar if we want one */
  if(pGFileList->Preference.statusbar == TRUE)
  {
    pGFileList->Statusbar = gtk_statusbar_new();
    gtk_box_pack_start(GTK_BOX(pGFileList),pGFileList->Statusbar,FALSE,TRUE,0);
    gtk_widget_show(pGFileList->Statusbar);    
  } else { pGFileList->Statusbar = NULL; }

  gtk_signal_connect(GTK_OBJECT(pGFileList->Clist),"event",
    GTK_SIGNAL_FUNC(gfilelist_event_cb), pGFileList);

  /* create a popup menu if wanted */
  /* for the "propertie" popup menu */
  if(pGFileList->Preference.pop_up_prop_menu == TRUE)
  {
     pGFileList->PopUpMenu = gtk_menu_new();
     
     menu_items = gtk_menu_item_new_with_label("Open");
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
     /* insert a line */
     menu_items = gtk_menu_item_new();
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
     /* insert a line */
     menu_items = gtk_menu_item_new();
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
		
     menu_items = gtk_menu_item_new_with_label("Properties...");
    gtk_signal_connect (GTK_OBJECT(menu_items), "activate",
	 GTK_SIGNAL_FUNC (prop_menu_properties_cb), pGFileList);
     gtk_widget_show(menu_items);
     gtk_menu_append(GTK_MENU (pGFileList->PopUpMenu), menu_items);
  }

  /* display the clist widget */
  gtk_widget_show (pGFileList->Clist);  
}
/**********************************************************/

/*************************************************************/
GtkWidget* gfilelist_new (GFileListPref pGFileListPref)
{
  gfilelistpref_copy(&current_GFileListPref,&pGFileListPref);
  return GTK_WIDGET ( gtk_type_new( gfilelist_get_type() ) );
}
/*************************************************************/

/*************************************************/
/* Free a GFileList				 */
void gfilelist_free(GFileList *pGFileList)
{
  /* remove all the thing that are in the clist */
  gfilelist_remove_all(pGFileList);
  /* if we have a popup menu, free it */
  if(pGFileList->Preference.pop_up_prop_menu)
    gtk_widget_destroy(GTK_WIDGET(pGFileList->PopUpMenu));
  /* free strings in the Prefence */
  g_free(pGFileList->Preference.name_title);
  g_free(pGFileList->Preference.owner_title);
  g_free(pGFileList->Preference.group_title); 
  g_free(pGFileList->Preference.rights_title);  
  g_free(pGFileList->Preference.size_title);
  g_free(pGFileList->Preference.last_access_date_title); 
  g_free(pGFileList->Preference.last_modif_date_title);
  g_free(pGFileList->Preference.last_status_change_date_title);
  /* free all the other widget */
  gtk_widget_destroy(GTK_WIDGET(pGFileList));
}
/*************************************************/

/**************************************************************/
/* create a string which represent the date. This string MUST */
/* BE FREE after use.                                         */
static gchar *gfilelist_date_2_string(GFileList *pGFileList,time_t date)
{ struct tm *vtm;
  gchar *date_string;

  date_string = g_malloc(11);
  vtm = localtime(&date);
  /*European notation*/
  sprintf(date_string,"%2u/%2u/%4u",vtm->tm_mday,vtm->tm_mon+1,vtm->tm_year+1900);
  if(vtm->tm_mday<10)  date_string[0] = '0';
  if(vtm->tm_mon+1<10) date_string[3] = '0';
  return(date_string);
}
/**************************************************************/

/******************************************************/
/* For local use only. This procedure create a string */
/* which correspond to the rights.		      */
/* string must be an array of 10 char		      */
static void gfile_list_rights_2_string(mode_t mode,gchar string[])
{
  /* owner rights */
  if(mode & S_IRUSR) string[0]='r';
  else string[0]='-';
  if(mode & S_IWUSR) string[1]='w';
  else string[1]='-';
  if(mode & S_IXUSR) string[2]='x';
  else string[2]='-';
  if(mode & S_ISUID)
    if(string[2] != '-')
      string[2]='s';
    else
      string[2]='S';

  /* owner group rights */
  if(mode & S_IRGRP) string[3]='r';
  else string[3]='-';
  if(mode & S_IWGRP) string[4]='w';
  else string[4]='-';
  if(mode & S_IXGRP) string[5]='x';
  else string[5]='-';
  if(mode & S_ISGID)
    if(string[5] != '-')
      string[5]='s';    
    else
      string[5]='S';

  /* all the world */
  if(mode & S_IROTH) string[6]='r';
  else string[6]='-';
  if(mode & S_IWOTH) string[7]='w';
  else string[7]='-';
  if(mode & S_IXOTH) string[8]='x';
  else string[8]='-';
  if(mode & S_ISVTX)
    if(string[8] != '-')
      string[8]='t';    
    else
      string[8]='T';
  string[9]=0; /* end the string */
}
/******************************************************/

static gboolean status_bar_refresh = TRUE;

/***********************************************************/
/* add an element to the list 				   */
/* Return : TRUE if it will be display, FALSE else.        */
/* THIS MUST BE THE ONLY WAY TO ADD AN ELEMENT TO THE LIST */
static gboolean select_added_file = FALSE; /* usefull for gfilelist_modify_file */
 			             /* if TRUE, the new added file is selected */
gboolean gfilelist_add_file(GFileList *pGFileList,FileData *pFileData)
{
  gchar *info[9];
  gchar temp_uid[10];    /* for the owner  */
  gchar temp_gid[10];    /* for the group  */
  gchar temp_rights[10]; /* for the rights */
  gchar temp_size[15];   /* for the size   */
  gchar *empty="";
  own *own;
  grp *grp;
  gint row;

  /* see if thoses kind of files are display */
  if((pGFileList->Preference.show_hidden_file == FALSE) && 
     (pFileData->name[0] == '.') &&
     strcmp(pFileData->name,".") && strcmp(pFileData->name,".."))
    return(FALSE);
  
  info[0]=empty; info[1]=empty; info[2]=empty; info[3]=empty;
  info[4]=empty; info[5]=empty; info[6]=empty; info[7]=empty;
  info[8]=empty;

  if(pGFileList->Preference.name_pos<255) /* display the file name */
    info[pGFileList->Preference.name_pos] = pFileData->name;

  if(pGFileList->Preference.owner_pos<255) /* display the owner */
  {
    own = owner_find_by_nb(pGFileList->Preference.owner_list,pFileData->stat.st_uid);
    if(own != NULL) info[pGFileList->Preference.owner_pos] = own->name;
    else {
      sprintf(temp_uid,"%u",pFileData->stat.st_uid);
      info[pGFileList->Preference.owner_pos] = temp_uid;
    }
  }

  if(pGFileList->Preference.group_pos<255)   /* display the group */
  {
    grp = group_find_by_nb(pGFileList->Preference.group_list,pFileData->stat.st_gid);
    if(grp != NULL) info[pGFileList->Preference.group_pos] = grp->name;
    else{
      sprintf(temp_gid,"%u",pFileData->stat.st_gid);
      info[pGFileList->Preference.group_pos] = temp_gid;
    }
  }
 
  if(pGFileList->Preference.rights_pos<255)  /* display the rights */
  {
    gfile_list_rights_2_string(pFileData->stat.st_mode,temp_rights);
    info[pGFileList->Preference.rights_pos] = temp_rights;
  }

  if(pGFileList->Preference.size_pos<255)    /* display the size */
  {
    sprintf(temp_size,"%u",pFileData->stat.st_size);
    info[pGFileList->Preference.size_pos] = temp_size;
  }

  if(pGFileList->Preference.last_access_date_pos<255)    /* display the create date */
  {
    info[pGFileList->Preference.last_access_date_pos] = 
      gfilelist_date_2_string(pGFileList,pFileData->stat.st_atime);
  }
  
  if(pGFileList->Preference.last_modif_date_pos<255)    /* display the modif date */
  {
    info[pGFileList->Preference.last_modif_date_pos] = 
      gfilelist_date_2_string(pGFileList,pFileData->stat.st_mtime);
  }

  if(pGFileList->Preference.last_status_change_date_pos<255)    /* display the last access date */
  {
    info[pGFileList->Preference.last_status_change_date_pos] = 
      gfilelist_date_2_string(pGFileList,pFileData->stat.st_ctime);
  }

  /* add to the list */
  row = gtk_clist_append(GTK_CLIST(pGFileList->Clist), info);
  /* take care of the icon */
  gtk_clist_set_pixtext(GTK_CLIST(pGFileList->Clist), row, 0, info[0],
                        5,pFileData->type->pixmap,
			pFileData->type->mask);
  /* attach the source information */
  gtk_clist_set_row_data(GTK_CLIST(pGFileList->Clist),row,
    		         pFileData);
			 
  /* select the added row if wanted */			 
  if(select_added_file)
    gtk_clist_select_row(GTK_CLIST(pGFileList->Clist),row,0);
			  
  /* free allocated strings */
  if(pGFileList->Preference.last_access_date_pos<255)
    g_free(info[pGFileList->Preference.last_access_date_pos]);

  if(pGFileList->Preference.last_modif_date_pos<255)
    g_free(info[pGFileList->Preference.last_modif_date_pos]);
            
  if(pGFileList->Preference.last_status_change_date_pos<255)
    g_free(info[pGFileList->Preference.last_status_change_date_pos]);

  /* auto_sort is not used because sort is made with attached datas
     and when an item is inserted, datas are not attached */
  gtk_clist_sort(GTK_CLIST(pGFileList->Clist));

  /* if wanted, we fill the statusbar. This can be avoid
     by setting status_bar_refresh = FALSE when adding many files */
  if((pGFileList->Statusbar) && (status_bar_refresh == TRUE))
  { gfilelist_fill_status(pGFileList); }

  return(TRUE);
}
/***********************************************************/

/*********************************************/
/* Add a file to the list by giving its name */
/* the file MUST be in the current viewed    */
/* directory. ex name = file1		     */
void gfilelist_add_file_by_name(GFileList *pGFileList,gchar *name)
{ FileData *vFileData;
  gchar *error_msg;
  GtkWidget *error_dialog;
  gint  index_row;
  FileData *vFileData_tmp;

  /* create a new FileData with info of the file "name"*/
  vFileData = file_data_new_from_file(pGFileList->current_path,name,
    pGFileList->Preference.file_type_list,pGFileList->Preference.owner_list);
  if(vFileData == NULL)
  {
    switch(gfile_errno)
    {
      case FILE_DATA_LSTAT_ERROR :
        error_msg = g_strconcat("ERROR while stating ",name,NULL);
	break;
      default :
        error_msg = g_strdup("Unknown ERROR");
    }
    error_dialog = gnome_message_box_new(error_msg,
      GNOME_MESSAGE_BOX_ERROR,GNOME_STOCK_BUTTON_OK,NULL);
    gtk_grab_add(error_dialog);
    gtk_signal_connect(GTK_OBJECT(error_dialog), "destroy",
      GTK_SIGNAL_FUNC(gtk_grab_remove),NULL);
    gtk_widget_show(error_dialog);
    g_free(error_msg);
    return;
  }

  /* remove an old file with the same name if exist */
  for(index_row=0;index_row<(GTK_CLIST(pGFileList->Clist))->rows;index_row++)
  {
    vFileData_tmp=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    /* if the content are the same */
    if((vFileData_tmp != NULL ) && (!strcmp(vFileData_tmp->name,vFileData->name)))
    { 
      /* remove the old one */
      gfilelist_remove_file(pGFileList,vFileData_tmp);
      /* add the new one */
      gfilelist_add_file(pGFileList,vFileData);
      return;
    }	
  }
  
  /* add to the list */
  gfilelist_add_file(pGFileList,vFileData);
}
/*********************************************/

/*******************************************************/
/* Del a file to the list by giving its full path name */
/* ex name = /dir1/subdir1/file1		       */
void gfilelist_remove_file_by_full_name(GFileList *pGFileList,gchar *full_name)
{ gchar *tmp_string;
  guint current_pos,copy_pos;
  gint  index_row;
  FileData *vFileData;
  
  if(full_name[0]!='/') return;

  tmp_string = g_malloc(strlen(full_name)+1);

  current_pos = 0;
  copy_pos = 0;
  while(full_name[current_pos] != 0 )
  { 
    tmp_string[current_pos] = full_name[current_pos];
    if(full_name[current_pos++] == '/') copy_pos = current_pos-1;
  }
  tmp_string[copy_pos+1] = 0; /* tmp_string = path */

  /* if it is in the current dir, we remove the file */
  if( !strcmp(tmp_string,pGFileList->current_path) )
  {
    for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
    {
      vFileData=(FileData *)gtk_clist_get_row_data(
        GTK_CLIST(pGFileList->Clist),index_row);
      /* if the content are the same */
      if((vFileData != NULL ) && (!strcmp(vFileData->name,&full_name[copy_pos+1])))
      { 
        gfilelist_remove_file(pGFileList,vFileData); 
	g_free(tmp_string);
        return;
      }	
    }
  }

  g_free(tmp_string);
}
/*******************************************************/

/*******************************************************/
/* Add a file to the list by giving its full path name */
/* ex name = /dir1/subdir1/file1		       */
void gfilelist_add_file_by_full_name(GFileList *pGFileList,gchar *full_name)
{ gchar *tmp_string;
  guint current_pos,copy_pos;
  
  if(full_name[0]!='/') return;

  tmp_string = g_malloc(strlen(full_name)+1);

  current_pos = 0;
  copy_pos = 0;
  while(full_name[current_pos] != 0 )
  { 
    tmp_string[current_pos] = full_name[current_pos];
    if(full_name[current_pos++] == '/') copy_pos = current_pos-1;
  }
  tmp_string[copy_pos+1] = 0; /* tmp_string = path */

  /*g_print("path : %s, name : %s.\n",tmp_string,&full_name[copy_pos+1]);*/

  /* if it is in the current dir, we add the file */
  if( !strcmp(tmp_string,pGFileList->current_path) )
    gfilelist_add_file_by_name(pGFileList,&full_name[copy_pos+1]);

  g_free(tmp_string);
}
/*******************************************************/

/***********************************************************/
/* add an element to the list 				   */
/* add it only if the element is in the current directory  */
/* a copy of pFileData will be used so we are free to do   */
/* what you want with pFileData after this called          */
void gfilelist_add_file_with_check(GFileList *pGFileList,FileData *pFileData)
{ FileData *vFileData;

  if((pGFileList == NULL) || (pFileData == NULL)) return;
  /* if it is not in the current path we have nothing to do */
  if( strcmp(pFileData->path,pGFileList->current_path) ) return;
  vFileData = file_data_copy(pFileData);
  gfilelist_add_file(pGFileList,vFileData);
}
/***********************************************************/

/***********************************************************/
/* return the row corresponding to pFileData. pFileData is */
/* find by comparing info (not pointer). If not found,     */
/* return -1.                                              */
gint gfilelist_find_row(GFileList *pGFileList,FileData *pFileData)
{ gint index_row;
  FileData *vFileData;

  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);

    if((vFileData != NULL ) && (file_data_is_equal(pFileData,
      vFileData)))
      return(index_row); 
  }
  return(-1);
}
/***********************************************************/

/************************************************************/
/* remove a file in the list using the FileData of the file */
/* The diff with gfilelist_remove_file is that we check if  */
/* the file really belong to the list.                      */
static gboolean unselect_remove_file = FALSE; /* usefull for gfilelist_modify_file */
   		                /* if TRUE, the file is unselected before removing */
static gboolean remove_file_was_selected = FALSE; /* let know if the file was */
                    /* selected or not (only if unselect_remove_file == TRUE) */
static gboolean remove_file_was_in_focus = FALSE;
void gfilelist_remove_file_with_check(GFileList *pGFileList,FileData *pFileData)
{
  gint index_row;
  FileData *vFileData;
  
  if((pGFileList == NULL) || (pFileData == NULL)) return;
  /* if it is not in the current path we have nothing to do */
  if( strcmp(pFileData->path,pGFileList->current_path) ) return;
  /* we make a check on data not pointer */
  /* find the corresponding file in our list */
  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    /* if the content are the same */ 
    if((vFileData != NULL ) && (file_data_is_equal(pFileData,
      vFileData)))
    { /* unselect file if wanted */
      if(unselect_remove_file == TRUE)
      {
        remove_file_was_selected = gfilelist_list_is_row_selected(pGFileList,index_row);
        remove_file_was_in_focus = 
	  (GTK_CLIST(pGFileList->Clist)->focus_row == index_row);
        if(remove_file_was_selected == TRUE)
	{
          gtk_clist_unselect_row(GTK_CLIST(pGFileList->Clist),index_row,0);
	}  
      }	
      gfilelist_remove_file(pGFileList,vFileData); 
      return; 
    }
  }
}
/******************************************************************/

/******************************************************************/
/* modify an entry in the list. The file corresponding to SrcFile */
/* is replaced by DstFile. SrcFile is found using informations    */
/* test (not pinter). A copy of DstFile is keeped.                */
void gfilelist_modify_file(GFileList *pGFileList,FileData *SrcFile,
  FileData *DstFile)
{ gboolean is_selected = FALSE;
  FileData *vFileData,*vNewFocusFileData;
  gint row_pos,last_focus_row;
  gboolean something_selected = TRUE;

  /* if the file we want to modify is not in the current path, exit */
  if(strcmp(pGFileList->current_path,SrcFile->path)) return;

  /* freeze the list to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileList->Clist));

  /* check if there is something selected in this list */
  if(GTK_CLIST(pGFileList->Clist)->selection == NULL) 
    something_selected = FALSE;
  /* if something was selected, keep the focus row item */
  if(something_selected)
  {
    last_focus_row = GTK_CLIST(pGFileList->Clist)->focus_row;
    vNewFocusFileData = gtk_clist_get_row_data(GTK_CLIST(pGFileList->Clist),last_focus_row);
  }
  
  row_pos = gfilelist_find_row(pGFileList,SrcFile);
  if(row_pos == -1) 
    g_print("GFILELIST, gfilelist_modify_file : SrcFile not found. name :%s%s.\n",
      SrcFile->path,SrcFile->name);
  if(something_selected)
  { is_selected = gfilelist_list_is_row_selected(pGFileList, row_pos);
    if(is_selected)
      select_added_file = TRUE;
  }    
  vFileData = file_data_copy(DstFile);    
  gfilelist_add_file(pGFileList,vFileData);
  select_added_file = FALSE;

  /* if focus row was the modified row, new focus row is the new modified row */
  if(something_selected)
    if(row_pos == last_focus_row)
      vNewFocusFileData = vFileData;

  /* if the modified row was selected, unselect it before removing */
  if(something_selected)
    if(is_selected)
      unselect_remove_file = TRUE;
  gfilelist_remove_file_with_check(pGFileList,SrcFile);
  unselect_remove_file = FALSE;

  /* if something was selected, set the new focus row */
  if(something_selected)
    GTK_CLIST(pGFileList->Clist)->focus_row = 
      gtk_clist_find_row_from_data(GTK_CLIST(pGFileList->Clist),vNewFocusFileData);

  /* unfreeze the list */
  gtk_clist_thaw(GTK_CLIST(pGFileList->Clist));
} 
/******************************************************************/

/**************************************************************/
/* remove an element to the list 			      */
/* THIS MUST BE THE ONLY WAY TO REMOVE AN ELEMENT TO THE LIST */
/* except the total destruction of the list                   */
void gfilelist_remove_file(GFileList *pGFileList,FileData *pFileData)
{ gint row_to_remove;
  /* if bad parameters we have nothing to do */
  if( (pGFileList==NULL) || (pFileData==NULL) ) return;
  /* try to find the row which correspond */
  row_to_remove = gtk_clist_find_row_from_data(GTK_CLIST(pGFileList->Clist),
                                               pFileData);
  if(row_to_remove != -1) /* if it existe */
    gtk_clist_remove(GTK_CLIST(pGFileList->Clist),row_to_remove);
  file_data_free(pFileData);
}
/**************************************************************/

/******************************************/
/* free all the file display in the clist */
void gfilelist_remove_all(GFileList *pGFileList)
{ gint index_row;
  FileData *vFileData;
  /* remove all the attached datas to the rows */
  for(index_row=0;index_row<(GTK_CLIST(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    if(vFileData != NULL ) file_data_free(vFileData);
  }
  /* remove the all row from the clist */
  gtk_clist_clear(GTK_CLIST(pGFileList->Clist));
}
/******************************************/

/***********************************/
/* Fill the statusbar of GFileList */
void gfilelist_fill_status(GFileList *pGFileList)
{ GList *select_row;
  gint index_row;
  FileData *vFileData;
  guint64 total_size = 0;
  guint32 nb_file = 0;
  gchar *to_display;
  /* if statusbar is not wanted, we have nothing to do here */
  if(pGFileList->Preference.statusbar == FALSE) return;
  
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  if(select_row == NULL) /* nothing selected, we display the total */
  { /* size of the file (directory are not counted). */
INFO_ON_ALL :
    for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
    {
      vFileData=(FileData *)gtk_clist_get_row_data(
        GTK_CLIST(pGFileList->Clist),index_row);
      /*g_print("fich : %s\n",vGFileListData->name);*/
      if(vFileData != NULL )
        if(S_ISREG(vFileData->stat.st_mode))
	{ nb_file++; total_size += vFileData->stat.st_size; }
    }
    to_display = g_malloc(27+20+10);
    sprintf(to_display," files : %u   total : %u bytes",nb_file,total_size);
  } else {
    /* 2 cases. 1 selected = we display more infos on the file */
    /* else we display the size of all the selected files. */
    if(select_row->next == NULL) 
    {
      vFileData=(FileData *)gtk_clist_get_row_data(
        GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
      if(vFileData == NULL) return; /* not possible*/
      /* special : if we are on . or .. we display info on all files */
      if(!strcmp(vFileData->name,".") ) goto INFO_ON_ALL;
      if(!strcmp(vFileData->name,"..") ) goto INFO_ON_ALL;

      if( vFileData->extra_info & FILE_DATA_IS_BROKEN_SYMB ){
        to_display = g_strconcat(" broken link to : ",vFileData->link_to_file,NULL);
      }else
      if( S_ISLNK(vFileData->stat.st_mode) ){
        to_display = g_strconcat(" link to : ",vFileData->link_to_file,NULL);
      }else
      if( S_ISDIR(vFileData->stat.st_mode) ){
        to_display = g_strconcat(" directory : ",vFileData->name,NULL);
      }else
      if(  S_ISSOCK(vFileData->stat.st_mode) ){
        to_display = g_strconcat(" socket : ",vFileData->name,NULL);
      }else
      if( S_ISBLK(vFileData->stat.st_mode) ){
        to_display = g_strconcat(" block device : ",vFileData->name,NULL);
      }else
      if( S_ISCHR(vFileData->stat.st_mode) ){
        to_display = g_strconcat(" character device : ",vFileData->name,NULL);
      }else
      if( S_ISFIFO(vFileData->stat.st_mode) ){
        to_display = g_strconcat(" fifo : ",vFileData->name,NULL);
      }else
      if( S_ISREG(vFileData->stat.st_mode) ){
        to_display = g_malloc(25+20+strlen(vFileData->name));
        sprintf(to_display," file : %s   size : %u bytes",
	  vFileData->name,vFileData->stat.st_size);
      }else{	  
        to_display = g_strdup("unknown file type");
      }

    } else { /* we display the size of all the selected files. */
      while(select_row)
      {
        vFileData=(FileData *)gtk_clist_get_row_data(
          GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
        /*g_print("fich : %s\n",vFileData->name);*/
        if(vFileData != NULL )
          if(S_ISREG(vFileData->stat.st_mode))
  	  { nb_file++; total_size += vFileData->stat.st_size; }
        select_row = select_row->next;
      }
      to_display = g_malloc(27+20+10);
      sprintf(to_display," files : %u   total : %u bytes",nb_file,total_size);
    } 
  }
  if ((GTK_STATUSBAR(pGFileList->Statusbar))->messages)
     gtk_statusbar_pop (GTK_STATUSBAR(pGFileList->Statusbar), 1);
  gtk_statusbar_push (GTK_STATUSBAR(pGFileList->Statusbar), 1,to_display);
  
  g_free(to_display);
}
/***********************************/

/*************************************************************/
/* Fill a GFileList with the content of the directory 'name' */
/* 'name' is the full path of the directory. DON'T ENDED by  */
/* a '/' (except for '/' directory)                          */
/* Return : TRUE = ok, FALSE = the directory doesn't exit    */

typedef struct
{
  GFileList    *vGFileList;
  FileDataList *vFileDataList;
} add_func_param;

gboolean add_func (FileData *pFileData,gpointer data)
{ 
  gfilelist_add_file(((add_func_param *)(data))->vGFileList,pFileData);
  file_data_list_append(((add_func_param *)(data))->vFileDataList,pFileData);
  /* the pFileData is keep at least for the FileDataList */
  return(TRUE);
}
/* if return is FALSE gfile_errno can be set to
FILE_DATA_DIR_EACCES   Permission denied.
FILE_DATA_DIR_EMFILE   Too many file descriptors in use by process.
FILE_DATA_DIR_ENFILE   Too many files are currently open in the system.
FILE_DATA_DIR_ENOENT   Directory does not exist, or name is an empty string.
FILE_DATA_DIR_ENOMEM   Insufficient memory to complete the operation.
FILE_DATA_DIR_ENOTDIR  name is not a directory.
FILE_DATA_DIR_UNKNOWN  unknown error.
*/
gboolean gfilelist_open_dir(GFileList *pGFileList,gchar *name)
{ gboolean return_value = TRUE;
  gchar *error_msg;
  GtkWidget *error_dialog;
  add_func_param *vadd_func_param;

  /* Freeze the clist to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileList->Clist));

  /* say that there is no row selected in the clist */
  pGFileList->Clist_last_selected_row = -1;

  /* Free the list before fill it again */
  gfilelist_remove_all(pGFileList);

  /* Free the current_path */
  if(pGFileList->current_path) g_free(pGFileList->current_path);
  /* write the new one */
  if(name[0]=='/' && name[1]==0) /* if we are at the root */
    pGFileList->current_path = g_strdup(name);
  else
    pGFileList->current_path = g_strconcat(name,"/",NULL);

  /* devalidate the status bar refresh to speed a bit */
  status_bar_refresh = FALSE;

  vadd_func_param = g_malloc(sizeof(add_func_param));
  vadd_func_param->vGFileList = pGFileList;
  vadd_func_param->vFileDataList = file_data_list_new();

  if(file_data_open_dir(name,pGFileList->Preference.file_type_list,
    pGFileList->Preference.owner_list,add_func,vadd_func_param) == FALSE)
  {
    return_value = FALSE;
    switch(gfile_errno)
    {
      case FILE_DATA_DIR_EACCES  : /*Permission denied.*/
        error_msg = g_strconcat("Permission denied to enter in ",name,NULL);
        break;      
      case FILE_DATA_DIR_EMFILE  : /*Too many file descriptors in use by process.*/
        error_msg = g_strdup("Too many file descriptors\nin use by process");
        break;
      case FILE_DATA_DIR_ENFILE  : /*Too many files are currently open in the system.*/
        error_msg = g_strdup("Too many files are currently\nopen in the system");
        break;
      case FILE_DATA_DIR_ENOENT  : /*Directory does not exist, or name is an empty string.*/
        error_msg = g_strdup("Directory does not exist");
        break;
      case FILE_DATA_DIR_ENOMEM  : /*Insufficient memory to complete the operation.*/
        error_msg = g_strdup("Insufficient memory to\ncomplete the operation");
        break;
      case FILE_DATA_DIR_ENOTDIR : /*name is not a directory.*/
        error_msg = g_strconcat(name,"\nis not a directory",NULL);
        break;
      case FILE_DATA_DIR_UNKNOWN : /* unknown error. */
        error_msg = g_strdup("Unknown ERROR (BUG?).");
        break;
    }
    error_dialog = gnome_message_box_new(error_msg,
      GNOME_MESSAGE_BOX_ERROR,GNOME_STOCK_BUTTON_OK,NULL);
    g_free(error_msg);
    gtk_grab_add(error_dialog);
    gtk_signal_connect(GTK_OBJECT(error_dialog), "destroy",
      GTK_SIGNAL_FUNC(gtk_grab_remove),NULL);
    gtk_widget_show(error_dialog);
  }
  
  status_bar_refresh = TRUE;

  /* Unfreeze the clist */
  gtk_clist_thaw(GTK_CLIST(pGFileList->Clist));

  /* we fill the statusbar */
  gfilelist_fill_status(pGFileList);

  /* reset the last selected row */
  pGFileList->Clist_last_selected_row = -1;

  /* alert that we have scan a directory */
  gtk_signal_emit(GTK_OBJECT(pGFileList),
    gfilelist_signals[SCAN_DIR_SIGNAL],vadd_func_param->vFileDataList);
  
  file_data_list_free(vadd_func_param->vFileDataList);
  g_free(vadd_func_param);
	 
  return(return_value);
}
/*************************************************************/

/* transform a string to go back to the parent directory */
static void down_dir(gchar *path,gchar *pathdest)
{
  guint lastchar = 0;
  guint16 compt;

  if(strcmp(path,"/")) /* if we are not a the root ("/") */
  {
     /* find */
     for(lastchar=0;(lastchar<2000)&& (path[lastchar]);lastchar++);
     lastchar-=2;
     for(;lastchar && (path[lastchar]!='/');lastchar--);
     /* make a copy of the good part */
     for(compt=0;compt<=lastchar;compt++) pathdest[compt]=path[compt];
     pathdest[lastchar+1]=0;
  } else { pathdest[0]='/'; pathdest[1]=0; }
}

/**********************************************************************/
/* procedure called each time there is something selected in the list */
static void gfilelist_row_select_cb(GtkCList *clist,gint row,gint column, 
  GdkEvent *event,GFileList *pGFileList)
{
  gchar *tempbuf,*temp_dir_name;
  FileData *vFileData;
  gchar *error_msg;
  GtkWidget *error_dialog;
  FileDataList *vFileDataList;
 
  /* do something only if the user has made a double click */
  if(row==pGFileList->Clist_last_selected_row)
  {
    /* look at the attached datas */
    vFileData = (FileData *)gtk_clist_get_row_data(
                GTK_CLIST(pGFileList->Clist),row);
				      
    /* if it is a directory (or a link on a directory)*/
    if( S_ISDIR(vFileData->stat.st_mode) ||  
        (S_ISLNK(vFileData->stat.st_mode) && 
	 !(vFileData->extra_info & FILE_DATA_IS_BROKEN_SYMB) &&
	 S_ISDIR(vFileData->link_stat->st_mode)) )
    {
       /*g_print("Directory path : %s, Name : %s\n",vFileData->path,
       						  vFileData->name);*/
       if( !strcmp(vFileData->name,".") ) /* reload the current directory */
       {
         tempbuf = g_strdup(vFileData->path);
	 /* if we are not at the root, suppress the last '/' */
	 if(tempbuf[1] != 0)
	   tempbuf[strlen(tempbuf)-1] = 0;
	 gfilelist_open_dir(pGFileList,tempbuf);
	 g_free(tempbuf);
       } else
       if( !strcmp(vFileData->name,"..") )  /* go to the parent directory */
       {
         tempbuf = g_malloc(strlen(vFileData->path)+1);
	 /* find the parent directory name */
	 down_dir(vFileData->path,tempbuf);
	 /* if we are not at the root, suppress the last '/' */
	 if(tempbuf[1] != 0)
	   tempbuf[strlen(tempbuf)-1] = 0;	 
	 gfilelist_open_dir(pGFileList,tempbuf);
	 g_free(tempbuf);
	 	 
	 /* emit the corresponding signal */
	 gtk_signal_emit(GTK_OBJECT(pGFileList),gfilelist_signals[DOWN_DIR_SIGNAL]);

       } else {	/* we go to the selected directory */
       
	 /* make a copy of the name of the directory we try to enter */
	 temp_dir_name = g_strdup(vFileData->name);

         tempbuf = g_strconcat(vFileData->path,vFileData->name,NULL);

	 if(gfilelist_open_dir(pGFileList,tempbuf))
	  /* let the others known that we go to the upper directory
	     temp_dir_name if we successful open if */
	   gtk_signal_emit(GTK_OBJECT(pGFileList),gfilelist_signals[UP_DIR_SIGNAL],
	                   temp_dir_name);
	 g_free(tempbuf);	 
	 g_free(temp_dir_name);
       }
    } else { /* this is not a directory */
      if(vFileData->type->program_str) /* if type defined*/
      {
/*        g_print("program : %s\n",vFileData->type->program_str);*/
	tempbuf = g_strconcat(vFileData->type->program_str," \"",
	  vFileData->path,vFileData->name,"\"&",NULL);
	/* WE CAN IMPROVE THAT */
	system(tempbuf);
	g_free(tempbuf);
      } else if(vFileData->extra_info & FILE_DATA_IS_ALLOW_TO_EXEC) {
	tempbuf = g_strconcat("\"",vFileData->path,vFileData->name,"\"&",NULL);
        /* go in the directory where the file is */
	if(chdir(vFileData->path))
	{
	  switch(errno)
	  {
	    case EFAULT :
	      error_msg = "path points outside your accessible address space";
	      break;
            case ENAMETOOLONG :
              error_msg = "path is too long";
	      break;
            case ENOENT :
	      error_msg = "The file does not exist";
	      break;
            case ENOMEM :
	      error_msg = "Insufficient kernel memory was available";
	      break;
            case ENOTDIR : 
	      error_msg = "A component of path is not a directory";
	      break;
            case EACCES :
	      error_msg = "Search  permission  is  denied  on  a component of path";
	      break;
            case ELOOP : 
	      error_msg = "Too  many  symbolic  links  were  encountered   in
               resolving path";
	      break;
            case EIO : 
	      error_msg = "An I/O error occurred";
	      break;
            default :
	      error_msg = "Unknown ERROR";
	      break;
	  }
	  error_dialog = gnome_message_box_new(error_msg,
          GNOME_MESSAGE_BOX_ERROR,GNOME_STOCK_BUTTON_OK,NULL);
          gtk_grab_add(error_dialog);
          gtk_signal_connect(GTK_OBJECT(error_dialog), "destroy",
            GTK_SIGNAL_FUNC(gtk_grab_remove),NULL);
          gtk_widget_show(error_dialog);

	} else /* if chdir = ok, exec the command */
	  system(tempbuf);
	g_free(tempbuf);
      }
    }
  } else {
    pGFileList->Clist_last_selected_row = row;
        /* update the statusbar*/
    if(pGFileList->Preference.statusbar)
      gfilelist_fill_status(pGFileList);
  }
}
/**********************************************************************/

/****************************************/
/* called each time a row is unselected */
static void gfilelist_row_unselect_cb(GtkCList *clist,gint row,gint column, GdkEvent
*event,GFileList *pGFileList)
{
  if(pGFileList->Preference.statusbar)
     gfilelist_fill_status(pGFileList);
  /* more than one selected => double click no more valid */
  pGFileList->Clist_last_selected_row = -1;   
}
/****************************************/

/************************************************/
/* Set a new FileType. The caller is responsible*/
/* for freeing the last one.                    */
void gfilelist_set_filetype(GFileList *pGFileList,FileTypeList *pFileTypeList)
{ gint index_row;
  FileData *vFileData;
  FileType *fich_type;
  gchar *text,*text_copy;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  guint8 spacing;
  
  pGFileList->Preference.file_type_list = pFileTypeList;
  
  /* Freeze the clist to avoid flicker */
  gtk_clist_freeze(GTK_CLIST(pGFileList->Clist));

  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  {
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    fich_type = file_data_find_type(pGFileList->Preference.file_type_list,vFileData);
    if(fich_type == NULL){gfile_error(FILE_TYPE_ERROR); exit(FILE_TYPE_ERROR);}

    /*g_print("change type for : %s\n",vFileData->name);*/
    gtk_clist_get_pixtext(GTK_CLIST(pGFileList->Clist),index_row,0,&text,
      			  &spacing,&pixmap,&mask);

    /*g_print("row 0 text : %s\n",text);*/
    text_copy = g_strdup(text);
    gtk_clist_set_pixtext(GTK_CLIST(pGFileList->Clist),index_row, 0, text_copy,
                          5,vFileData->type->pixmap,
			  vFileData->type->mask);
    g_free(text_copy);
  }

  /* Unfreeze the clist */
  gtk_clist_thaw(GTK_CLIST(pGFileList->Clist));
}
/************************************************/

/*******************************************************/
/* create a list of all the selected file in the clist */
/* when we are called.				       */
/* return : The list.                                  */
FileDataList *gfilelist_list_selected(GFileList *pGFileList)
{ GList *select_row;
  FileData *vFileData;
  FileDataList *vFileDataList;
  
  vFileDataList = file_data_list_new();

  /* place select_row on the selected line list */
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  
  while(select_row)
  { /* get the data attach to the selected line */
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),(gint)select_row->data);
    /* add the selected data to our list */
    file_data_list_append(vFileDataList, vFileData);
    /* go to the next selected line */
    select_row = g_list_next(select_row);
  }
  return(vFileDataList);
}
/*******************************************************/

/*******************************************/
/* create a list of all file in the clist. */
/* return : the list.	                   */
FileDataList *gfilelist_list_all(GFileList *pGFileList)
{
  FileData *vFileData;
  FileDataList *vFileDataList;
  gint index_row;
  
  vFileDataList = file_data_list_new();

  for(index_row=0;index_row<((GtkCList *)(pGFileList->Clist))->rows;index_row++)
  { /* get the data attach to the selected line */
    vFileData=(FileData *)gtk_clist_get_row_data(
      GTK_CLIST(pGFileList->Clist),index_row);
    /* add the file data to our list */
    file_data_list_append(vFileDataList, vFileData);
  }
  return(vFileDataList);
}
/*******************************************************/


/*******************************************************/
/* Asked if a row is seleted or not                    */
/* return : TRUE if row is selected, FALSE else.       */
gboolean gfilelist_list_is_row_selected(GFileList *pGFileList, gint row)
{ GList *select_row;
  
  /* place select_row on the selected line list */
  select_row = (GList *)(GTK_CLIST(pGFileList->Clist)->selection);
  
  while((select_row) && ((gint)select_row->data != row))
  {
    /* go to the next selected line */
    select_row = g_list_next(select_row);
  }
  return(select_row != NULL);  
}
/*******************************************************/

/************************************************************/
/* use this if you want to unselect every thing in the list */
void gfilelist_unselect_all(GFileList *pGFileList)
{
  gtk_signal_emit_by_name(GTK_OBJECT(pGFileList->Clist),"unselect_all");
  /* reset the last selected row */
  pGFileList->Clist_last_selected_row = -1;
}
/************************************************************/

static void prop_file_changed_cb(GtkWidget *Widget,FileData *SrcFile,
  FileData *DstFile,GFileList *pGFileList)
{
  gtk_signal_emit(GTK_OBJECT(pGFileList),
    gfilelist_signals[FILE_MODE_CHANGED_SIGNAL],SrcFile,DstFile);
}  

static void prop_menu_properties_cb(GtkWidget *Widget,GFileList *pGFileList)
{ FileDataList *vFileDataList;
  GtkWidget *propreties;

  vFileDataList = gfilelist_list_selected(pGFileList);
  
  propreties = file_properties_new(vFileDataList,pGFileList->Preference.file_type_list,
    pGFileList->Preference.owner_list);
  gtk_widget_show(propreties);

  gtk_signal_connect(GTK_OBJECT(propreties),"file_mode_changed",
    GTK_SIGNAL_FUNC(prop_file_changed_cb),pGFileList);

  file_data_list_free(vFileDataList);
}

/****************************************/
/* for the popup menu                   */
/* ________ UNDER DEVELOPEMENT _________*/  
static gint gfilelist_event_cb(GtkCList *clist, GdkEvent *event,
  GFileList *pGFileList)
{ gint row,column;
  FileDataList *vFileDataList;

  if(event->type == GDK_KEY_PRESS) {
    return(key_press_event_cb(NULL,&(event->key),pGFileList));
  } else if(event->type == GDK_BUTTON_PRESS) {
    GdkEventButton *bevent = (GdkEventButton *) event; 
    /* if it is the third button */
    if(bevent->button == 3)
    { /*g_print("properties called.\n");*/
      if(pGFileList->Preference.pop_up_prop_menu == FALSE) return(FALSE);

      row = bevent->x; column = bevent->y;		      
      gtk_clist_get_selection_info(GTK_CLIST(pGFileList->Clist),
        row,column,&row,&column);
      /*g_print("row : %d, col : %d\n",row,column);*/

      /* do something only if the user right click on a selected file */
      if(gfilelist_list_is_row_selected(pGFileList,row) == TRUE)
      {
        vFileDataList = gfilelist_list_selected(pGFileList);
        gtk_menu_popup(GTK_MENU(pGFileList->PopUpMenu), NULL, NULL, NULL, NULL,
                       bevent->button, bevent->time);
/*        pop_menu_new(NULL,bevent->button, bevent->time);*/
      } 
      /* Tell calling code that we have handled this event; the buck
      * stops here. */	      
      return TRUE;      
    }  	      
    /* Tell calling code that we have not handled this event; pass it on. */
    return FALSE;
  }
  /* Tell calling code that we have not handled this event; pass it on. */
  return FALSE;
}
/****************************************/

/*************************************************************/
/* pFileDataList is the new content of the current directory */
void gfilelist_update(GFileList *pGFileList, FileDataList *pFileDataList)
{ FileDataList *vFileDataList;
  FileDataList *vFileDataList2;
  GList *node,*node2,*node_tmp;
  
  vFileDataList  = gfilelist_list_all(pGFileList);
  vFileDataList2 = file_data_list_copy(pFileDataList);

  node2 = vFileDataList2->list->next;
  while(node2)
  {
    node = vFileDataList->list->next;
    while((node) && (!file_data_is_equal(node->data,node2->data)))
    { node = node->next; }
    
    if(node)
    { /* the item is present in the both list, modify it */
      gfilelist_modify_file(pGFileList,node->data,node2->data);
      /* suppress the node2 in vFileDataList2 */
      vFileDataList2->list->next = node2->next;
      if(node2->next) node2->next->prev = vFileDataList2->list;
      file_data_free(node2->data);
      g_free(node2);
      /* suppress the node in vFileDataList */      
      node->prev->next = node->next;
      if(node->next) node->next->prev = node->prev;
      file_data_free(node->data);
      g_free(node);
    } else {
      /* the item is only present in the new list */
      if(gfilelist_add_file(pGFileList,node2->data) == FALSE)
        file_data_free(node2->data);
      /* suppress the node2 in vFileDataList2 */
      vFileDataList2->list->next = node2->next;
      if(node2->next) node2->next->prev = vFileDataList2->list;
      g_free(node2);
    }
    node2 = vFileDataList2->list->next;
  }

  /* remove all we still have in the vFileDataList */
  node = vFileDataList->list->next;
  while(node)
  {
    /* I can use node->data because the item in the clist use
       the same FileData than node->data (because of ref_count system) */
    gfilelist_remove_file(pGFileList,node->data);

    /* suppress the node in vFileDataList */      
    node_tmp = node->next;
    node->prev->next = node->next;
    if(node->next) node->next->prev = node->prev;
    file_data_free(node->data);
    g_free(node);

    node = node_tmp;
  }

  file_data_list_free(vFileDataList);
  file_data_list_free(vFileDataList2);  
}
/*************************************************************/
