/****************************************/
/*  PreferenceFileType.c body           */
/*					*/
/*  Description : The file type part of */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

/**********************************************************************/
/* TO DO                                                              */
/* - improve drag & drop (don't use focus_row because when nothing is */
/* selected as src but dst is correct focus_row is correct and a move */
/*   is done)                                                         */
/**********************************************************************/

guint preference_file_type_addedit_get_type ();
GtkWidget *preference_file_type_new(GtkWidget *parent_window,FileTypeList *to_use_file_type);

static void addedit_destroy_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit, PreferenceFileType *pPreferenceFileType);
static void addedit_add_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,FileType *beforetype, FileType *newtype,
  PreferenceFileType *pPreferenceFileType);  
static void addedit_remove_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,FileType *removetype,
  PreferenceFileType *pPreferenceFileType);
static void file_type_add_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_edit_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_remove_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_move_up_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);
static void file_type_move_down_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType);

static void preference_file_type_class_init(PreferenceFileTypeClass *pPreferenceFileTypeClass);
static void preference_file_type_init(PreferenceFileType *pPreferenceFileType);
static void destroy_cb(GtkWidget *GtkWidget,PreferenceFileType *pPreferenceFileType);

static void target_drag_data_received(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,GtkSelectionData *data,guint info,guint time,PreferenceFileType *pPreferenceFileType);
static gboolean target_drag_drop(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceFileType *pPreferenceFileType);
static gboolean target_drag_motion(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceFileType *pPreferenceFileType);
static void source_drag_data_get(GtkWidget *widget,GdkDragContext *context,
  GtkSelectionData *selection_data,guint info,guint time,gpointer data);

/*************************************************************/
guint preference_file_type_get_type ()
{
  static guint preference_file_type_type = 0;

  if (!preference_file_type_type)
  {
    GtkTypeInfo preference_file_type_info =
    {
       "PreferenceFileType",
       sizeof (PreferenceFileType),
       sizeof (PreferenceFileTypeClass),
       (GtkClassInitFunc)  preference_file_type_class_init,
       (GtkObjectInitFunc) preference_file_type_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_file_type_type = gtk_type_unique(gtk_hbox_get_type(), &preference_file_type_info);
  }
  return preference_file_type_type;
}
/*************************************************************/

/*************************************************************/
static void preference_file_type_class_init(PreferenceFileTypeClass *class)
{
  /* here we create the new signals if we want */    
}
/*************************************************************/
    
/********************************************************/
/* Remove the selected type from the clist and from the */
/* file type list if it is not a special type.          */
static void file_type_remove_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ FileType *tmp_type;
  gint vselect_row;

  /* if there is something selected */
  if(pPreferenceFileType->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
  else return;

  tmp_type = gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row);
  /* if the one we want to remove is in edition, exit */
  if((pPreferenceFileType->addedit) &&
     (PREFERENCE_FILE_TYPE_ADDEDIT(pPreferenceFileType->addedit)->lasttype == tmp_type)) return;

  gtk_clist_remove(pPreferenceFileType->list,vselect_row);
  /* remove the file type */
  file_type_list_remove_file_type(pPreferenceFileType->to_use_file_type,tmp_type);

  /*to be sure all needed types are still present (even if they are invisible)*/
  file_type_list_add_needed_file_type(pPreferenceFileType->to_use_file_type);
}
/********************************************************/

/*********************************/
static void addedit_destroy_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit, PreferenceFileType *pPreferenceFileType)
{
  gtk_widget_set_sensitive(pPreferenceFileType->add,TRUE);
  gtk_widget_set_sensitive(pPreferenceFileType->edit,TRUE);
  gtk_widget_set_sensitive(pPreferenceFileType->remove,TRUE);  
  pPreferenceFileType->addedit = NULL; 
}
/*********************************/

/*****************************************************/
static void addedit_add_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,FileType *beforetype, FileType *newtype,
  PreferenceFileType *pPreferenceFileType)
{ gchar *entry[2];
  gint vselect_row;

/*  g_print("addedit_add_type_cb called. newtype string : %s, icon : %s\n",
    newtype->string,newtype->icon);*/
  if(newtype->mode == USER_DEFINED)
    entry[0] = newtype->type_str;
  else
    entry[0] = mode_str[newtype->mode];
  entry[1] = newtype->program_str;

  if(beforetype == NULL) {
    vselect_row = gtk_clist_append(pPreferenceFileType->list,entry);
  } else { 
    vselect_row = gtk_clist_find_row_from_data(pPreferenceFileType->list,
      beforetype);
    vselect_row = gtk_clist_insert(pPreferenceFileType->list,
      vselect_row+1,entry);
  }
  
  gtk_clist_set_pixtext(pPreferenceFileType->list, vselect_row,
  		        0, entry[0],5,
			newtype->pixmap,
			newtype->mask);
  /* attach the datas for making a relation between this list and the */
  /* existing file types */
  gtk_clist_set_row_data(pPreferenceFileType->list,vselect_row,newtype);
}
/*****************************************************/

/******************************************************************/
static void addedit_remove_type_cb(PreferenceFileTypeAddedit
  *pPreferenceFileTypeAddedit,FileType *removetype,
  PreferenceFileType *pPreferenceFileType)
{ gint vselect_row;

  g_print("addedit_remove_type_cb called\n");
  vselect_row = gtk_clist_find_row_from_data(pPreferenceFileType->list,
    removetype);
  gtk_clist_remove(pPreferenceFileType->list,vselect_row);
  /* to be sure all needed types are still present */
  file_type_list_add_needed_file_type(pPreferenceFileType->to_use_file_type);
}
/******************************************************************/

/**************************/
/* add a type to the list */
static void file_type_add_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ FileType *beforetype;
  gint vselect_row;

  /* only one dialog open at a time */
  if(pPreferenceFileType->addedit) return;

  gtk_widget_set_sensitive(pPreferenceFileType->add,FALSE);
  gtk_widget_set_sensitive(pPreferenceFileType->edit,FALSE);

  /* if there is something selected, add after that type */
  if(pPreferenceFileType->list->selection != NULL){
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
    beforetype = gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row);
  /* else add at the end */
  } else beforetype = NULL;

  pPreferenceFileType->addedit = preference_file_type_addedit_new
    (pPreferenceFileType->to_use_file_type,beforetype,NULL);

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "add_type",
                      GTK_SIGNAL_FUNC (addedit_add_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "remove_type",
                      GTK_SIGNAL_FUNC (addedit_remove_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "destroy",
                      GTK_SIGNAL_FUNC (addedit_destroy_cb),
                      pPreferenceFileType);

  gtk_widget_show(pPreferenceFileType->addedit);
}
/**************************/

/***************************/
/* edit a type to the list */
void file_type_edit_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ FileType *lasttype;
  gint vselect_row;

  /* only one dialog open at a time */
  if(pPreferenceFileType->addedit) return;

  /* if there is something selected, add after that type */
  if(pPreferenceFileType->list->selection != NULL){
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
    lasttype = gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row);
  /* else add at the end */
  } else return;

  gtk_widget_set_sensitive(pPreferenceFileType->add,FALSE);
  gtk_widget_set_sensitive(pPreferenceFileType->edit,FALSE);
  gtk_widget_set_sensitive(pPreferenceFileType->remove,FALSE);

  pPreferenceFileType->addedit = preference_file_type_addedit_new
    (pPreferenceFileType->to_use_file_type,lasttype,lasttype);

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "add_type",
                      GTK_SIGNAL_FUNC (addedit_add_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "remove_type",
                      GTK_SIGNAL_FUNC (addedit_remove_type_cb),
                      pPreferenceFileType);  

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType->addedit), "destroy",
                      GTK_SIGNAL_FUNC (addedit_destroy_cb),
                      pPreferenceFileType);

  gtk_widget_show(pPreferenceFileType->addedit);
}
/**************************/

/***************************/
/* move the select type up */
void file_type_move_up_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ gint vselect_row;

  /* if there is something selected */
  if(pPreferenceFileType->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
  else return;  
  /*g_print("row to move up : %d.\n");*/
  if(vselect_row > 0)
  {
    gtk_clist_freeze (pPreferenceFileType->list);
    gtk_clist_unselect_row (pPreferenceFileType->list,vselect_row,0);
    gtk_clist_swap_rows(pPreferenceFileType->list,vselect_row-1,vselect_row);
    gtk_clist_select_row (pPreferenceFileType->list,vselect_row-1,0);
    gtk_clist_thaw   (pPreferenceFileType->list);
    /* move up in the file type too */
    /* swap in the file_type list */
    file_type_list_swap_file_type(pPreferenceFileType->to_use_file_type,
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row),
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row-1));
  }  
}
/***************************/

/*****************************/
/* move the select type down */
void file_type_move_down_cb(GtkWidget *widget,PreferenceFileType *pPreferenceFileType)
{ gint vselect_row;

  /* if there is something selected */
  if(pPreferenceFileType->list->selection != NULL)
    vselect_row = (gint)((GList *)(pPreferenceFileType->list->selection)->data);
  else return;  

  if(vselect_row < (pPreferenceFileType->list->rows - 1) )
  {
    gtk_clist_freeze (pPreferenceFileType->list);
    gtk_clist_unselect_row (pPreferenceFileType->list,vselect_row,0);
    gtk_clist_swap_rows(pPreferenceFileType->list,vselect_row,vselect_row+1);
    gtk_clist_select_row (pPreferenceFileType->list,vselect_row+1,0);
    gtk_clist_thaw   (pPreferenceFileType->list);    
    /* move down in the file type too */
    /* swap in the file_type list */
    file_type_list_swap_file_type(pPreferenceFileType->to_use_file_type,
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row),
      gtk_clist_get_row_data(pPreferenceFileType->list,vselect_row+1));
  }  
}
/*****************************/

/************************************************/
/* Create a widget for the preference file type */
/* Return : the widget			        */

static char * up_xpm[] = {
"14 11 5 1",
" 	c None",
".	c #C6C6C6",
"+	c #969696",
"@	c #000000",
"#	c #FFFFFF",
"      .+      ",
"     .@@+     ",
"    .@@@@+    ",
"   .@@@@@@+   ",
"  .@@@@@@@@+  ",
" .@@@@@@@@@@+ ",
".@@@@@@@@@@@@.",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".#############"};

static char * down_xpm[] = {
"14 11 5 1",
" 	c None",
".	c #C6C6C6",
"+	c #969696",
"@	c #000000",
"#	c #FFFFFF",
".++++++++++++.",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
".@@@@@@@@@@@@#",
" .@@@@@@@@@@# ",
"  .@@@@@@@@#  ",
"   .@@@@@@#   ",
"    .@@@@#    ",
"     .@@#     ",
"      .#      "};


/* use as parameters given by preference_file_type_new to 
   preference_file_init */
static FileTypeList  *current_to_use_file_type;
static GtkWidget *current_parent_window;

/***********************************/
/* call when the widget is destroy */
static void destroy_cb(GtkWidget *Widget,PreferenceFileType *pPreferenceFileType)
{
  /* if there is an addedit dialog open, close it */
  if(pPreferenceFileType->addedit)
    gnome_dialog_close(GNOME_DIALOG(pPreferenceFileType->addedit));
}
/***********************************/

/**************************************************/
/* called when a FileType is selected in the list */
static void select_row_cb(GtkCList *clist,gint row,gint column,GdkEvent *event,
  PreferenceFileType *pPreferenceFileType)
{
  if((pPreferenceFileType->addedit != NULL) &&
     (PREFERENCE_FILE_TYPE_ADDEDIT(pPreferenceFileType->addedit)->lasttype == 
       (FileType *)gtk_clist_get_row_data(pPreferenceFileType->list,row)))
    gtk_widget_set_sensitive(pPreferenceFileType->remove,FALSE);
}
/**************************************************/

/****************************************************/
/* called when a FileType is unselected in the list */
static void unselect_row_cb(GtkCList *clist,gint row,gint column,
  GdkEvent *event,PreferenceFileType *pPreferenceFileType)
{
  if(GTK_WIDGET_IS_SENSITIVE(pPreferenceFileType->remove) == FALSE)
    gtk_widget_set_sensitive(pPreferenceFileType->remove,TRUE);  
}
/****************************************************/

enum {
  TARGET_FILE_TYPE
};

static GtkTargetEntry target_table[] = {
  { "FileType", 0, TARGET_FILE_TYPE }
};

static guint n_targets = sizeof(target_table) / sizeof(target_table[0]);


void preference_file_type_init(PreferenceFileType *pPreferenceFileType)
{
  GtkWidget *buttons;
  GtkWidget *move_up;
  GtkWidget *move_down;
  GtkWidget *scrolled_widget;
  gchar     *titles[2] = {"Type","Program"};
  FileType  *tmptype = NULL;
  /* __ FOR THE UP & DOWN PIXMAP ____*/
  GdkPixmap *updown_pixmap;
  GdkBitmap *updown_mask;
  GtkWidget *updown_pixmap_widget;

  /* take in account parameters */
  pPreferenceFileType->to_use_file_type = current_to_use_file_type;

  gtk_box_set_homogeneous(GTK_BOX(pPreferenceFileType),FALSE);
  gtk_box_set_spacing(GTK_BOX(pPreferenceFileType),0);

  gtk_signal_connect (GTK_OBJECT(pPreferenceFileType), "destroy",
                      GTK_SIGNAL_FUNC(destroy_cb),
                      pPreferenceFileType);

  buttons = gtk_vbox_new (FALSE, 0);

  pPreferenceFileType->edit = gtk_button_new_with_label ("Edit");
  gtk_widget_show(pPreferenceFileType->edit);
  gtk_box_pack_start (GTK_BOX (buttons), pPreferenceFileType->edit, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pPreferenceFileType->edit), "clicked",
                      GTK_SIGNAL_FUNC (file_type_edit_cb),
                      pPreferenceFileType);

  pPreferenceFileType->add = gtk_button_new_with_label ("Add");
  gtk_widget_show(pPreferenceFileType->add);
  gtk_box_pack_start (GTK_BOX (buttons), pPreferenceFileType->add, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pPreferenceFileType->add), "clicked",
                      GTK_SIGNAL_FUNC (file_type_add_cb),
                      pPreferenceFileType);


  pPreferenceFileType->remove = gtk_button_new_with_label ("Remove");
  gtk_widget_show(pPreferenceFileType->remove);
  gtk_box_pack_start (GTK_BOX (buttons), pPreferenceFileType->remove, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (pPreferenceFileType->remove), "clicked",
                      GTK_SIGNAL_FUNC (file_type_remove_cb),
                      pPreferenceFileType);
  /*** create the up button ***/
  updown_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&updown_mask,NULL,up_xpm);

  /* create a widget with the pixmap */
  updown_pixmap_widget = gtk_pixmap_new(updown_pixmap,updown_mask);
  /* free the pixmap */
  gdk_pixmap_unref(updown_pixmap); gdk_bitmap_unref(updown_mask);

  move_up = gtk_button_new ();
  gtk_container_add(GTK_CONTAINER(move_up),updown_pixmap_widget);
  gtk_widget_show(updown_pixmap_widget);
  gtk_widget_show(move_up);
  gtk_box_pack_start (GTK_BOX (buttons), move_up, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (move_up), "clicked",
                      GTK_SIGNAL_FUNC (file_type_move_up_cb),
                      pPreferenceFileType);
		      
  /*** create the down button ***/
  updown_pixmap = gdk_pixmap_create_from_xpm_d (
    current_parent_window->window,&updown_mask,NULL,down_xpm);

  /* create a widget with the pixmap */
  updown_pixmap_widget = gtk_pixmap_new(updown_pixmap,updown_mask);
  /* free the pixmap */
  gdk_pixmap_unref(updown_pixmap); gdk_bitmap_unref(updown_mask);

  move_down = gtk_button_new();
  gtk_container_add(GTK_CONTAINER(move_down),updown_pixmap_widget);
  gtk_widget_show(updown_pixmap_widget);
  gtk_widget_show(move_down);
  gtk_box_pack_start (GTK_BOX (buttons), move_down, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (move_down), "clicked",
                      GTK_SIGNAL_FUNC (file_type_move_down_cb),
                      pPreferenceFileType);

  pPreferenceFileType->list = GTK_CLIST(gtk_clist_new_with_titles(2,titles));

  gtk_signal_connect (GTK_OBJECT (pPreferenceFileType->list), "select_row",
                      GTK_SIGNAL_FUNC (select_row_cb),
                      pPreferenceFileType);

  gtk_signal_connect (GTK_OBJECT (pPreferenceFileType->list), "unselect_row",
                      GTK_SIGNAL_FUNC (unselect_row_cb),
                      pPreferenceFileType);

  gtk_clist_set_row_height (pPreferenceFileType->list, 18);
  gtk_clist_set_column_width(pPreferenceFileType->list,0,220);

  gtk_widget_show (GTK_WIDGET(pPreferenceFileType->list));

  /* we fill with all the existing file types */
  file_type_list_init(pPreferenceFileType->to_use_file_type);
  while(file_type_list_is_end(pPreferenceFileType->to_use_file_type) == FALSE)
  {
    /* add at the end if not an internal needed type */
    if(file_type_list_current(pPreferenceFileType->to_use_file_type)->icon_name)
      addedit_add_type_cb(NULL,NULL, file_type_list_current(
        pPreferenceFileType->to_use_file_type),pPreferenceFileType);

    file_type_list_next(pPreferenceFileType->to_use_file_type);
  }

  /* define the scrolled policy */
  scrolled_widget = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_widget),
    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  gtk_widget_show(scrolled_widget);
  gtk_container_add(GTK_CONTAINER(scrolled_widget), GTK_WIDGET(
    pPreferenceFileType->list));
    
  gtk_box_pack_start (GTK_BOX(pPreferenceFileType),scrolled_widget,TRUE,TRUE,0);
  gtk_box_pack_start (GTK_BOX(pPreferenceFileType), buttons, FALSE, TRUE, 0);

  gtk_container_border_width (GTK_CONTAINER(pPreferenceFileType), 4);
  gtk_container_border_width (GTK_CONTAINER(buttons), 4);

  gtk_widget_show(buttons);
  /* no addedit dialog open at the startup */
  pPreferenceFileType->addedit = NULL; 
  
  /* for the drag&drop */
  /* set the list as a destination */
  gtk_drag_dest_set(GTK_WIDGET(pPreferenceFileType->list), 0, 
    target_table, n_targets, 0);

  gtk_signal_connect(GTK_OBJECT(pPreferenceFileType->list),"drag_motion",
    GTK_SIGNAL_FUNC(target_drag_motion),pPreferenceFileType);

  gtk_signal_connect(GTK_OBJECT(pPreferenceFileType->list),"drag_data_received",
    GTK_SIGNAL_FUNC(target_drag_data_received),pPreferenceFileType);

  gtk_signal_connect(GTK_OBJECT(pPreferenceFileType->list),"drag_drop",
    GTK_SIGNAL_FUNC(target_drag_drop),pPreferenceFileType);
  /* set the list as a source */
  gtk_drag_source_set(GTK_WIDGET(pPreferenceFileType->list),GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
    target_table, n_targets, GDK_ACTION_COPY | GDK_ACTION_MOVE);

  gtk_signal_connect(GTK_OBJECT(pPreferenceFileType->list),"drag_data_get",
    GTK_SIGNAL_FUNC(source_drag_data_get),pPreferenceFileType);
}
/************************************************/

/*****************************************************/
/* Create a new widget for the preference file type  */
/* Return : the widget			             */
GtkWidget *preference_file_type_new(GtkWidget *parent_window,FileTypeList *to_use_file_type)
{
  current_parent_window = parent_window;
  current_to_use_file_type = to_use_file_type;

  return GTK_WIDGET (gtk_type_new( preference_file_type_get_type() ));
}
/*****************************************************/

/*******************************************/
/* UNDER DEVELOPPEMENT FOR THE DRAG & DROP */
/* not done                                */
/*******************************************/

gint start_row;

static gboolean target_drag_motion(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceFileType *pPreferenceFileType)
{ 
  GtkWidget *source_widget;
  gint column;
  source_widget = gtk_drag_get_source_widget(context);
  
  /*g_print("motion called.\n");*/
  if((GTK_WIDGET(pPreferenceFileType->list) == source_widget) &&
     (pPreferenceFileType->list->rows))
  {
    /*gtk_clist_get_selection_info(pPreferenceFileType->list,x,
      y-pPreferenceFileType->list->column_title_area.height,&start_row,&column);
    g_print("(%d,%d) row = %d\n",x,y,start_row);*/
    gdk_drag_status (context, context->suggested_action, time);
    return TRUE;
  } else return FALSE;
}

static gboolean target_drag_drop(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,guint time,PreferenceFileType *pPreferenceFileType)
{
  /*g_print("drop called.\n");*/

  if((context->targets) && (pPreferenceFileType->list->rows))
  {
    gtk_drag_get_data(widget,context,GPOINTER_TO_INT(context->targets->data),time);
    return TRUE;
  }
  return FALSE;
}

static void target_drag_data_received(GtkWidget *widget,GdkDragContext *context,
  gint x,gint y,GtkSelectionData *data,guint info,guint time,PreferenceFileType *pPreferenceFileType)
{
  GtkWidget *source_widget;
  gint row,column,focus_row;

  /*g_print("drag_received called at (%d,%d).\n",x,y);*/
  source_widget = gtk_drag_get_source_widget(context);
  /*if(source_widget == NULL) g_print("source_widget NULL.\n");
  if(info != TARGET_FILE_TYPE) g_print("info != TARGET_FILE_TYPE.\n");
  if(pPreferenceFileType == NULL) g_print("pPreferenceFileType NULL.\n");
  if(GTK_WIDGET(pPreferenceFileType->list) != source_widget) g_print("pPreferenceFileType->list != source_widget.\n");
  */
  if((info == TARGET_FILE_TYPE) && (source_widget) && 
     (GTK_WIDGET(pPreferenceFileType->list) == source_widget) &&
     (pPreferenceFileType->list->rows != 0))
  {
    /*if(gtk_clist_get_selection_info(pPreferenceFileType->list,x,
      y-pPreferenceFileType->list->column_title_area.height,&row,&column))
      g_print("out selection.\n");*/

    gtk_clist_get_selection_info(pPreferenceFileType->list,x,
      y-pPreferenceFileType->list->column_title_area.height,&row,&column);

    if((row >= pPreferenceFileType->list->rows) || (row < 0))
    {
      gtk_drag_finish (context, FALSE, FALSE, time);
      return;
    }

    focus_row = pPreferenceFileType->list->focus_row;
    /*g_print("we take the DROP in account. line %d to %d\n",focus_row,row);*/

    file_type_list_move_file_type(pPreferenceFileType->to_use_file_type,
      gtk_clist_get_row_data(pPreferenceFileType->list,focus_row),
      gtk_clist_get_row_data(pPreferenceFileType->list,row));

    gtk_clist_row_move(pPreferenceFileType->list,focus_row,row);

    gtk_drag_finish (context, TRUE, FALSE, time);
  } else {
    /*g_print("the DROP go to TRASH\n");*/
    gtk_drag_finish (context, FALSE, FALSE, time);
  }
}


static void source_drag_data_get(GtkWidget *widget,GdkDragContext *context,
  GtkSelectionData *selection_data,guint info,guint time,gpointer data)
{
  /*g_print("get_data called.\n");*/
  if(info == TARGET_FILE_TYPE)
    gtk_selection_data_set(selection_data,selection_data->target,
      8, "FileType swapping", 17);
}
