/****************************************/
/*  PreferenceFileTypeAddEdit.c body    */
/*					*/
/*  Description : Bouton Add and Edit of*/
/*	          the file type part of */
/*		  the preference dialog	*/
/*					*/
/*  Written by : Daniel Lacroix	        */
/*  GFile is under the GPL2 licence     */
/****************************************/

#include "GFile.h"

guint preference_file_type_addedit_get_type ();

void preference_file_type_addedit_ok_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

void preference_file_type_addedit_cancel_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

GtkWidget *preference_file_type_addedit_new(FileTypeList *pFileTypeList,
  FileType *beforetype, FileType *lasttype);

static void preference_file_type_addedit_class_init(
  PreferenceFileTypeAddeditClass *pPreferenceFileTypeAddeditClass);

static void preference_file_type_addedit_init(
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

static void destroy_cb(GtkWidget *Widget, 
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit);

/*************************************************************/
guint preference_file_type_addedit_get_type ()
{
  static guint preference_file_type_addedit_type = 0;

  if (!preference_file_type_addedit_type)
  {
    GtkTypeInfo preference_file_type_addedit_info =
    {
       "PreferenceFileTypeAddedit",
       sizeof (PreferenceFileTypeAddedit),
       sizeof (PreferenceFileTypeAddeditClass),
       (GtkClassInitFunc)  preference_file_type_addedit_class_init,
       (GtkObjectInitFunc) preference_file_type_addedit_init,
       (GtkArgSetFunc) NULL,
       (GtkArgGetFunc) NULL
     };
     preference_file_type_addedit_type = gtk_type_unique(gnome_dialog_get_type(), &preference_file_type_addedit_info);
  }
  return preference_file_type_addedit_type;
}
/*************************************************************/

/*************************************************************/
enum { ADD_TYPE_SIGNAL, REMOVE_TYPE_SIGNAL, LAST_SIGNAL};
static gint preference_file_type_addedit_signals[LAST_SIGNAL] = { 0 };

static void one_file_type_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*one_file_type_marshal_func)(PreferenceFileTypeAddedit *,
      FileType *, gpointer);
    one_file_type_marshal_func rfunc = (one_file_type_marshal_func) func;
    (*rfunc) (PREFERENCE_FILE_TYPE_ADDEDIT(object),
	      GTK_VALUE_POINTER(args[0]), func_data);
}

static void two_file_type_marshal(GtkObject *object, GtkSignalFunc func, gpointer func_data, GtkArg *args) {
    typedef void (*two_file_type_marshal_func)(PreferenceFileTypeAddedit *,
      FileType *, FileType *, gpointer);
    two_file_type_marshal_func rfunc = (two_file_type_marshal_func) func;
    (*rfunc) (PREFERENCE_FILE_TYPE_ADDEDIT(object),
	      GTK_VALUE_POINTER(args[0]),GTK_VALUE_POINTER(args[1]), func_data );
}

static void preference_file_type_addedit_class_init(PreferenceFileTypeAddeditClass *class)
{
  /* here we create the new signals if we want */
  GtkObjectClass *object_class;
  object_class = (GtkObjectClass*) class;

  preference_file_type_addedit_signals[ADD_TYPE_SIGNAL] = gtk_signal_new
            ("add_type",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceFileTypeAddeditClass,add_type),
            two_file_type_marshal, GTK_TYPE_NONE, 2,GTK_TYPE_POINTER,
	    GTK_TYPE_POINTER);    

  preference_file_type_addedit_signals[REMOVE_TYPE_SIGNAL] = gtk_signal_new
            ("remove_type",
            GTK_RUN_FIRST,object_class->type,
            GTK_SIGNAL_OFFSET (PreferenceFileTypeAddeditClass,remove_type),
            one_file_type_marshal, GTK_TYPE_NONE, 1,GTK_TYPE_POINTER);    
	    
   gtk_object_class_add_signals(object_class, 
     preference_file_type_addedit_signals, LAST_SIGNAL);
   class->add_type = NULL;  class->remove_type = NULL;	    
}
/*************************************************************/

/* use as parameter given by preference_file_type_addedit_new
  to preference_file_type_addedit_init */
static FileType *current_preference_file_type_addedit_beforetype;
static FileType *current_preference_file_type_addedit_lasttype;
static FileTypeList *current_preference_file_type_addedit_to_use_file_type_list;

static gchar *suppress_start(gchar *start,gchar *string)
{ guint16 pos = 0;
  guint16 end_pos;
  /*g_print("Start : %s, String : %s\n",start,string);*/

  while ((start[pos]) && (string[pos]) && (start[pos] == string[pos])){pos++;}
  /* if we are not at the end of start, that mean that string doesn't
     start like the "start" string. */  
  if(start[pos] != 0) {gfile_error(ICON_NOT_IN_PATH); exit(ICON_NOT_IN_PATH);}
  /* else, we check if there is not '/' in &string[pos] */
  end_pos = pos;
  while(string[end_pos])
  { if(string[end_pos++] == '/') {gfile_error(ICON_NOT_IN_PATH); exit(ICON_NOT_IN_PATH);} 
  }
  return(&string[pos]);
}

/*******************************************************************/
/* called each time the icon file is changed using the icon button */
void icon_change_cb(IconList *pIconList, gchar *icon_file,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  if(icon_file != NULL)
    gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),
      suppress_start(file_type_get_icon_path(),icon_file));
  else /* no icon selected */
    gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),"");
}
/*******************************************************************/

struct _ModeChoose{
  PreferenceFileTypeAddedit *vPreferenceFileTypeAddedit;
  guint8 mode;
};

typedef struct _ModeChoose ModeChoose;

/*********************************************************************/
/* call when the mode is changed (USER_DEFINED, NORMAL_DIRECTORY...) */
void mode_change_cb(GtkMenuShell *menu_shell, ModeChoose *pModeChoose)
{ FileType *vFileType;
/*  g_print("menu change. new is : %s.\n",mode_str[pModeChoose->mode]);*/
  pModeChoose->vPreferenceFileTypeAddedit->current_mode = pModeChoose->mode;

  if(pModeChoose->mode == USER_DEFINED)
  {
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->string),1);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->string_label),1);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->program),1);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->program_label),1);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->browse),1);
  } else {
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->string),0);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->string_label),0);    
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->program),0);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->program_label),0);
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->browse),0);
    
    gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->is_link),1);
    vFileType = file_type_list_find_file_type(pModeChoose->vPreferenceFileTypeAddedit->to_use_file_type,
       pModeChoose->mode,FALSE);
    if(vFileType && vFileType->icon_name)
    {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pModeChoose->vPreferenceFileTypeAddedit->is_link),1);
      gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->is_link),0);
    } else {
      vFileType = file_type_list_find_file_type(pModeChoose->vPreferenceFileTypeAddedit->to_use_file_type,
        pModeChoose->mode,TRUE);
      if(vFileType && vFileType->icon_name)
      {
        gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(pModeChoose->vPreferenceFileTypeAddedit->is_link),0);
        gtk_widget_set_sensitive(GTK_WIDGET(pModeChoose->vPreferenceFileTypeAddedit->is_link),0);
      }
    }
  }
}
/*********************************************************************/

/******************************************/
/* call when the browse button is pressed */
void browse_cancel_cb(GtkWidget *Widget,PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{ gtk_widget_destroy(pPreferenceFileTypeAddedit->browse_dialog); }

void browse_destroy_cb(GtkWidget *Widget,PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{ pPreferenceFileTypeAddedit->browse_dialog = NULL; }

void browse_ok_cb(GtkWidget *Widget,PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{ gchar *string;
  
  string = gtk_file_selection_get_filename(GTK_FILE_SELECTION(pPreferenceFileTypeAddedit->browse_dialog));
  /* if it is not a directory, we take it */
  if(string[strlen(string)-1] != '/')
  {
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pPreferenceFileTypeAddedit->program)->entry),string);
    gtk_widget_destroy(pPreferenceFileTypeAddedit->browse_dialog);
  } 
}

void program_browse_cb(GtkButton *button,PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  if(pPreferenceFileTypeAddedit->browse_dialog == NULL)
  {
    pPreferenceFileTypeAddedit->browse_dialog = gtk_file_selection_new("Choose the program");
    gtk_widget_show(pPreferenceFileTypeAddedit->browse_dialog);

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(pPreferenceFileTypeAddedit->browse_dialog)->ok_button),
      "clicked",GTK_SIGNAL_FUNC(browse_ok_cb),pPreferenceFileTypeAddedit);
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(pPreferenceFileTypeAddedit->browse_dialog)->cancel_button),
      "clicked",GTK_SIGNAL_FUNC(browse_cancel_cb),pPreferenceFileTypeAddedit);
    gtk_signal_connect(GTK_OBJECT(pPreferenceFileTypeAddedit->browse_dialog),
      "destroy",GTK_SIGNAL_FUNC(browse_destroy_cb),pPreferenceFileTypeAddedit);
  }
}
/******************************************/

/****************************************/
/* Create a new dialog for add or edit. */
static void preference_file_type_addedit_init(
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  GtkWidget *h1;
  GtkWidget *label;
  GtkWidget *menu;
  GtkWidget *menu_item;
  gint i;
  ModeChoose *vModeChoose;
  GtkWidget  *table;
  GtkWidget  *icon_button;
  GList      *strings = NULL;
  GList      *tmp_node;
  FileType   *tmptype;
  ModeChoose vModeChoose2;
  gchar      *string;

  /* use parameters */
  pPreferenceFileTypeAddedit->beforetype =
    current_preference_file_type_addedit_beforetype;  
  pPreferenceFileTypeAddedit->lasttype = 
    current_preference_file_type_addedit_lasttype;
  pPreferenceFileTypeAddedit->to_use_file_type = 
    current_preference_file_type_addedit_to_use_file_type_list;

  /* set the right title */
  if(pPreferenceFileTypeAddedit->lasttype == NULL)
    gtk_window_set_title(GTK_WINDOW(pPreferenceFileTypeAddedit),"New file type");
  else
    gtk_window_set_title(GTK_WINDOW(pPreferenceFileTypeAddedit),"Edit file type");

  /* add the OK and CANCEL buttons */
  gnome_dialog_append_buttons(GNOME_DIALOG(pPreferenceFileTypeAddedit),
    GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);

  /* Cancel button */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceFileTypeAddedit),1,
    GTK_SIGNAL_FUNC(preference_file_type_addedit_cancel_cb),pPreferenceFileTypeAddedit);
  /* Ok button */
  gnome_dialog_button_connect(GNOME_DIALOG(pPreferenceFileTypeAddedit),0,
    GTK_SIGNAL_FUNC(preference_file_type_addedit_ok_cb),pPreferenceFileTypeAddedit);

  /* connect the destroy signal of the dialog to destroy_cb */
  gtk_signal_connect(GTK_OBJECT(pPreferenceFileTypeAddedit), "destroy",
    GTK_SIGNAL_FUNC(destroy_cb),pPreferenceFileTypeAddedit);    

  h1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show(h1);
  gtk_container_border_width (GTK_CONTAINER(h1), 4);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox),
    h1, FALSE, FALSE, 0);

  /* create the option menu (USER_DEFINED ...) */  
  pPreferenceFileTypeAddedit->mode_list = gtk_option_menu_new();
  gtk_widget_show(pPreferenceFileTypeAddedit->mode_list);
  gtk_box_pack_start (GTK_BOX (h1), pPreferenceFileTypeAddedit->mode_list, 
    FALSE, FALSE, 0);

  menu = gtk_menu_new();
  gtk_widget_show(menu);
  
  gtk_option_menu_set_menu(GTK_OPTION_MENU(pPreferenceFileTypeAddedit->mode_list),menu);
  pPreferenceFileTypeAddedit->change_mode_struct = g_slist_alloc();

  for(i = NORMAL_DIRECTORY; i<LAST_MUST_BE_TYPE; i++)
  {
    menu_item = gtk_menu_item_new_with_label(mode_str[i]);
    vModeChoose = g_malloc(sizeof(ModeChoose));
    vModeChoose->vPreferenceFileTypeAddedit = pPreferenceFileTypeAddedit;
    vModeChoose->mode = i;
    gtk_signal_connect(GTK_OBJECT(menu_item),"activate",
      GTK_SIGNAL_FUNC(mode_change_cb),vModeChoose);
    g_slist_append(pPreferenceFileTypeAddedit->change_mode_struct,vModeChoose);
    gtk_widget_show(menu_item);
    gtk_menu_append(GTK_MENU(menu),menu_item);
    /* if this type is totally defined (not internally), devalidate it */
    if(i!=USER_DEFINED)
    {
      tmptype  = file_type_list_find_file_type(pPreferenceFileTypeAddedit->to_use_file_type,i,FALSE);
      if(tmptype && tmptype->icon_name)
      {
        tmptype = file_type_list_find_file_type(pPreferenceFileTypeAddedit->to_use_file_type,i,TRUE);
        if(tmptype && tmptype->icon_name)
	  gtk_widget_set_sensitive(GTK_WIDGET(menu_item),0); 
      }
    }
  }
  for(i = LAST_MUST_BE_TYPE+1; i<=USER_DEFINED; i++)
  {
    menu_item = gtk_menu_item_new_with_label(mode_str[i]);
    vModeChoose = g_malloc(sizeof(ModeChoose));
    vModeChoose->vPreferenceFileTypeAddedit = pPreferenceFileTypeAddedit;
    vModeChoose->mode = i;
    gtk_signal_connect(GTK_OBJECT(menu_item),"activate",
      GTK_SIGNAL_FUNC(mode_change_cb),vModeChoose);
    g_slist_append(pPreferenceFileTypeAddedit->change_mode_struct,vModeChoose);      
    gtk_widget_show(menu_item);
    gtk_menu_append(GTK_MENU(menu),menu_item);
    /* if this type is totally defined, devalidate it excepted USER_DEFINED */
    if(file_type_list_find_file_type(pPreferenceFileTypeAddedit->to_use_file_type,i,FALSE) &&
       file_type_list_find_file_type(pPreferenceFileTypeAddedit->to_use_file_type,i,TRUE))
      gtk_widget_set_sensitive(GTK_WIDGET(menu_item),0);
  }

  gtk_widget_set_usize(GTK_WIDGET(pPreferenceFileTypeAddedit->mode_list),216,10);
  /* set old mode if new or USER_DEFINED else */
  if(pPreferenceFileTypeAddedit->lasttype != NULL) {
    if(pPreferenceFileTypeAddedit->lasttype->mode < LAST_MUST_BE_TYPE)
      gtk_option_menu_set_history(
        GTK_OPTION_MENU(pPreferenceFileTypeAddedit->mode_list),
        pPreferenceFileTypeAddedit->lasttype->mode);
    else
      gtk_option_menu_set_history(
        GTK_OPTION_MENU(pPreferenceFileTypeAddedit->mode_list),
        pPreferenceFileTypeAddedit->lasttype->mode-1);
    vModeChoose2.mode = pPreferenceFileTypeAddedit->lasttype->mode;
  } else {
    gtk_option_menu_set_history(
      GTK_OPTION_MENU(pPreferenceFileTypeAddedit->mode_list),USER_DEFINED-1);
    vModeChoose2.mode = USER_DEFINED;
  }

  /* the link check button */
  pPreferenceFileTypeAddedit->is_link = gtk_check_button_new_with_label("link");
  gtk_widget_show(pPreferenceFileTypeAddedit->is_link);
  gtk_box_pack_start(GTK_BOX (h1), pPreferenceFileTypeAddedit->is_link, 
    FALSE, FALSE, 4);    

  /* all the rest in a table */
  table = gtk_table_new(3,2,FALSE);
  gtk_widget_show(table);
  gtk_table_set_row_spacings(GTK_TABLE(table),8);
  gtk_table_set_col_spacings(GTK_TABLE(table),8);
  gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(pPreferenceFileTypeAddedit)->vbox),
    table, FALSE, TRUE, 0);

  /* type string */
  pPreferenceFileTypeAddedit->string_label = gtk_label_new("type string");
  gtk_widget_show(pPreferenceFileTypeAddedit->string_label);
  gtk_table_attach_defaults(GTK_TABLE(table),
    pPreferenceFileTypeAddedit->string_label,0,1,0,1);
  pPreferenceFileTypeAddedit->string = gtk_entry_new();
  gtk_widget_show(pPreferenceFileTypeAddedit->string);
  gtk_table_attach_defaults(GTK_TABLE(table),pPreferenceFileTypeAddedit->string,
    1,2,0,1);

  /* Program */
  pPreferenceFileTypeAddedit->program_label=gtk_label_new("program");
  gtk_widget_show(pPreferenceFileTypeAddedit->program_label);
  gtk_table_attach_defaults(GTK_TABLE(table),
    pPreferenceFileTypeAddedit->program_label,0,1,1,2);
  h1 = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(h1);
  gtk_table_attach_defaults(GTK_TABLE(table),h1,1,2,1,2);
  pPreferenceFileTypeAddedit->program = gtk_combo_new();
/*  gtk_entry_set_editable(GTK_ENTRY(pPreferenceFileTypeAddedit->program),TRUE);*/
  gtk_widget_show(pPreferenceFileTypeAddedit->program);
  /* fill the program combo list with all the all ready used program */  
  file_type_list_init(pPreferenceFileTypeAddedit->to_use_file_type);
  while(file_type_list_is_end(pPreferenceFileTypeAddedit->to_use_file_type) == FALSE)
  {
    if(file_type_list_current(pPreferenceFileTypeAddedit->to_use_file_type)->program_str)
    { 
      /* try to find this program in the all ready known list */
      for(tmp_node = strings; tmp_node && 
        strcmp(tmp_node->data,file_type_list_current(
	pPreferenceFileTypeAddedit->to_use_file_type)->program_str); 
        tmp_node = g_list_next(tmp_node));
      /* if new add it */
      if(tmp_node == NULL)
        strings = g_list_append(strings,
          file_type_list_current(pPreferenceFileTypeAddedit->to_use_file_type)->program_str);
    }
    file_type_list_next(pPreferenceFileTypeAddedit->to_use_file_type);
  }
  if(strings != NULL)
  {
    gtk_combo_set_popdown_strings(GTK_COMBO(pPreferenceFileTypeAddedit->program),
      strings);
    g_list_free(strings);
  }
  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pPreferenceFileTypeAddedit->program)->entry),"");
  gtk_box_pack_start(GTK_BOX(h1),pPreferenceFileTypeAddedit->program,TRUE,TRUE,0);
  pPreferenceFileTypeAddedit->browse = gtk_button_new_with_label("browse...");
  gtk_widget_show(pPreferenceFileTypeAddedit->browse);
  pPreferenceFileTypeAddedit->browse_dialog = NULL;
  gtk_signal_connect(GTK_OBJECT(pPreferenceFileTypeAddedit->browse),"clicked",
    GTK_SIGNAL_FUNC(program_browse_cb),pPreferenceFileTypeAddedit);    
  gtk_box_pack_start(GTK_BOX(h1),pPreferenceFileTypeAddedit->browse,FALSE,FALSE,4);

  /* Icon */
  label = gtk_label_new("icon");
  gtk_widget_show(label);
  gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
  h1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show(h1);
  gtk_table_attach_defaults(GTK_TABLE(table),h1,1,2,2,3);
  pPreferenceFileTypeAddedit->icon = gtk_entry_new();
/*  gtk_entry_set_editable(GTK_ENTRY(pPreferenceFileTypeAddedit->icon), TRUE);*/
  gtk_widget_show(pPreferenceFileTypeAddedit->icon);
  gtk_box_pack_start (GTK_BOX (h1), pPreferenceFileTypeAddedit->icon, TRUE, TRUE, 0);
  /* the icon button */
  if((pPreferenceFileTypeAddedit->lasttype == NULL) ||
     (pPreferenceFileTypeAddedit->lasttype->icon_name == NULL))
    icon_button = icon_list_new(NULL,file_type_get_icon_path());
  else {
    if(pPreferenceFileTypeAddedit->lasttype->icon_name[0] != '/') {
      string = g_strconcat(file_type_get_icon_path(),
        pPreferenceFileTypeAddedit->lasttype->icon_name,NULL);
      icon_button = icon_list_new(string,file_type_get_icon_path());
      g_free(string);
    } else
      icon_button = icon_list_new(pPreferenceFileTypeAddedit->lasttype->icon_name,
        file_type_get_icon_path());
  }
  gtk_signal_connect(GTK_OBJECT (icon_button), "icon_change",
    GTK_SIGNAL_FUNC (icon_change_cb),pPreferenceFileTypeAddedit);
  gtk_widget_show(icon_button);
  gtk_box_pack_start (GTK_BOX (h1), icon_button, FALSE, FALSE, 4);

  /* fill with the old values if we are in edit mode */
  if(pPreferenceFileTypeAddedit->lasttype)
  {
    if(pPreferenceFileTypeAddedit->lasttype->type_str)
      gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->string),pPreferenceFileTypeAddedit->lasttype->type_str);
    if(pPreferenceFileTypeAddedit->lasttype->program_str)
      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(pPreferenceFileTypeAddedit->program)->entry),pPreferenceFileTypeAddedit->lasttype->program_str);
    if(pPreferenceFileTypeAddedit->lasttype->icon_name)
      gtk_entry_set_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon),pPreferenceFileTypeAddedit->lasttype->icon_name);
  }
  /* to set sensitive or not type string & program */
  vModeChoose2.vPreferenceFileTypeAddedit = pPreferenceFileTypeAddedit;
  mode_change_cb(NULL, &vModeChoose2);

  /* set the old mode for link check button if edit mode */
  if(pPreferenceFileTypeAddedit->lasttype != NULL)
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
      pPreferenceFileTypeAddedit->is_link),
      pPreferenceFileTypeAddedit->lasttype->is_link);
}
/****************************************/

/***************************************************************/
/* Create a dialog for add or edit. If lasttype = NULL then it */
/* is in add mode else, in edit mode.                          */
/* If beforetype = NULL, insert at the end.                    */
GtkWidget *preference_file_type_addedit_new(FileTypeList *pFileTypeList,
  FileType *beforetype, FileType *lasttype)
{
  current_preference_file_type_addedit_beforetype = beforetype;
  current_preference_file_type_addedit_lasttype = lasttype;
  current_preference_file_type_addedit_to_use_file_type_list = pFileTypeList;

  return GTK_WIDGET(gtk_type_new( preference_file_type_addedit_get_type() ));
}
/***************************************************************/

static void destroy_dialog_cb(GtkWidget *Widget,GnomeDialog *pGnomeDialog)
{ gtk_grab_remove(GTK_WIDGET(pGnomeDialog)); }

static void preference_file_type_addedit_error(gchar *message)
{ GtkWidget *message_dialog;

  message_dialog = gnome_message_box_new(message,GNOME_MESSAGE_BOX_ERROR,
    GNOME_STOCK_BUTTON_OK,NULL);
    
  gtk_signal_connect (GTK_OBJECT(message_dialog), "destroy",
                      GTK_SIGNAL_FUNC (destroy_dialog_cb),
                      message_dialog);
  gtk_widget_show(message_dialog);
  gtk_grab_add(message_dialog);
}

/*************************************/
/* Button ok pressed in the Add mode */
void preference_file_type_addedit_ok_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{
  gchar     *entry[2];
  FileType *new_type,*vFileType;
  gboolean  is_link;
  gchar     *icon_name;
  gchar     *program_str;
  gchar     *type_str;
  guint8    mode;
  
  gchar     *tmp_string;
  gint      pos;
  GdkPixmap *pixmap; GdkBitmap *mask;

  /* mode */
  mode = pPreferenceFileTypeAddedit->current_mode;
  
  /* link or not */
  is_link = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
    pPreferenceFileTypeAddedit->is_link));

  /* type string */
  type_str = gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->string));
  for(pos = 0; type_str[pos] && ((type_str[pos]==' ')||(type_str[pos]=='\t'));pos++);
  if(mode == USER_DEFINED) {
    if(type_str[pos] == 0)
    {
      preference_file_type_addedit_error("you MUST give a \nreconization pattern.");
      return;
    }
  } else {
    type_str = NULL;
  }
  
  /* program string */
  program_str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(pPreferenceFileTypeAddedit->program)->entry));
  for(pos = 0; program_str[pos] && ((program_str[pos]==' ')||(program_str[pos]=='\t'));pos++);
  if((mode != USER_DEFINED) || (program_str[pos] == 0)) program_str = NULL;

  /* icon name */
  icon_name = gtk_entry_get_text(GTK_ENTRY(pPreferenceFileTypeAddedit->icon));
  for(pos = 0; icon_name[pos] && ((icon_name[pos]==' ')||(icon_name[pos]=='\t'));pos++);
  if(icon_name[pos] == 0)
  {
    preference_file_type_addedit_error("you MUST choose an icon.");
    return;
  }

  /* we create the new type */
  file_type_list_find_icon(pPreferenceFileTypeAddedit->to_use_file_type,
    icon_name,&pixmap,&mask);
/*  g_print("mode : %s, type : %s, pgm : %s, icon : %s, link : %d, pix : %X,"\
    " mask : %X\n",
    mode_str[mode],type_str,program_str,icon_name,is_link,pixmap,mask);*/
  new_type = file_type_new(mode,type_str,program_str,icon_name,is_link,
    pixmap,mask);
  
  if(new_type != NULL)
  {
    /* check if there is an internal must be present type corresponding
       and remove it (no more needed) */
    if(mode <  LAST_MUST_BE_TYPE)
    {
      vFileType = file_type_list_find_file_type(pPreferenceFileTypeAddedit->to_use_file_type,
        mode,is_link);
      if(vFileType && !vFileType->icon_name)
        file_type_list_remove_file_type(
          pPreferenceFileTypeAddedit->to_use_file_type,vFileType);
    }
    file_type_list_insert_file_type(pPreferenceFileTypeAddedit->to_use_file_type,
      pPreferenceFileTypeAddedit->beforetype, new_type);
	
    /* alerte the parent that we have inserted a new type */
    gtk_signal_emit(GTK_OBJECT(pPreferenceFileTypeAddedit),
      preference_file_type_addedit_signals[ADD_TYPE_SIGNAL],
	pPreferenceFileTypeAddedit->beforetype,
	new_type);

    /* if we are in edit mode and the new creation has worked */
    if(pPreferenceFileTypeAddedit->lasttype) { /* we are in edit mode */
      /* alerte the parent that we are going to suppress a type */
      gtk_signal_emit(GTK_OBJECT(pPreferenceFileTypeAddedit),
        preference_file_type_addedit_signals[REMOVE_TYPE_SIGNAL],
	  pPreferenceFileTypeAddedit->lasttype);
      /* remove the type */
      file_type_list_remove_file_type(pPreferenceFileTypeAddedit->to_use_file_type,
        pPreferenceFileTypeAddedit->lasttype);
    }
    gnome_dialog_close(GNOME_DIALOG(pPreferenceFileTypeAddedit));

  } else { /* we must alerte the user that the icon doesn't exist in */
           /* "icon_path"icons directory */ 
    if(gfile_errno == FILE_TYPE_LOADING_ICON_ERROR)
      tmp_string = g_strconcat("The icon : ",icon_name,",\n can't be found",NULL);
    else if(gfile_errno == FILE_TYPE_UNKOW_MUST_BE_FILE_TYPE)
      tmp_string = g_strdup("FILE_TYPE_UNKOW_MUST_BE_FILE_TYPE");    
    else if(gfile_errno == FILE_TYPE_PARAM_ERROR)
      tmp_string = g_strdup("FILE_TYPE_PARAM_ERROR");
    else
      tmp_string = g_strdup("Unexpected ERROR (BUG)");
    preference_file_type_addedit_error(tmp_string);
    g_free(tmp_string);
  }
}
/*************************************/

/*************************************************/
/* Button cancel pressed in the Add or Edit mode */
void preference_file_type_addedit_cancel_cb(GtkWidget *widget,
  PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{ gnome_dialog_close (GNOME_DIALOG(pPreferenceFileTypeAddedit)); }
/*************************************************/

/**************************************/
/* call when the dialog box is closed */
static void destroy_cb(GtkWidget *Widget, PreferenceFileTypeAddedit *pPreferenceFileTypeAddedit)
{ GSList *tmp_node;

  /* free the structures used has info for change mode */
  tmp_node = pPreferenceFileTypeAddedit->change_mode_struct->next;
  while(tmp_node != NULL)
  {
    g_free(tmp_node->data);
    tmp_node = g_slist_next(tmp_node);
  }
  g_slist_free(pPreferenceFileTypeAddedit->change_mode_struct);

  /* if the browse program dialog is still open, close it */
  if(pPreferenceFileTypeAddedit->browse_dialog)
    gtk_widget_destroy(pPreferenceFileTypeAddedit->browse_dialog);
}
/**************************************/
